@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.34 2000/09/20 20:06:11 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.36 2001/10/14 17:46:42 william Exp $
  */
 
 #define _INCLUDE_FROM_ATTR_C_
@@ -1388,8 +1388,12 @@ TgMenu *CreateAttrMenu(parent_menu, x, y, num_items, menu_strings,
    if (stMenuInfo.items == NULL) FailAllocMessage();
    memset(stMenuInfo.items, 0, (num_items+1)*sizeof(TgMenuItemInfo));
    for (item_info=stMenuInfo.items, i=0; i < num_items; item_info++, i++) {
-      item_info->menu_str = UtilStrDup(menu_strings[i]);
-      if (item_info->menu_str == NULL) FailAllocMessage();
+      if (menu_strings[i] == TGMUITEM_SEPARATOR) {
+         item_info->menu_str = TGMUITEM_SEPARATOR;
+      } else {
+         item_info->menu_str = UtilStrDup(menu_strings[i]);
+         if (item_info->menu_str == NULL) FailAllocMessage();
+      }
       if (status_strings != NULL && status_strings[i] != NULL) {
          item_info->status_str = UtilStrDup(status_strings[i]);
          if (item_info->status_str == NULL) FailAllocMessage();
@@ -1457,7 +1461,9 @@ void FreeEditAttrInfo(pEditAttrInfo)
    for (i=0; i < num_attrs; i++) {
       if (attr_names != NULL) UtilFree(attr_names[i]);
       if (attr_values != NULL) UtilFree(attr_values[i]);
-      if (attr_strings != NULL) UtilFree(attr_strings[i]);
+      if (attr_strings != NULL) {
+         if (attr_strings[i] != TGMUITEM_SEPARATOR) UtilFree(attr_strings[i]);
+      }
       if (status_strings != NULL) UtilFree(status_strings[i]);
    }
    if (attr_names != NULL) free(attr_names);
