@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/choice.c,v 4.41 2000/07/24 17:15:36 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/choice.c,v 4.43 2001/08/06 21:38:57 william Exp $
  */
 
 #define _INCLUDE_FROM_CHOICE_C_
@@ -2073,6 +2073,8 @@ int ModeEventHandler(input)
          SetCurChoice(mode);
          /* the raw status_str is English */
          SetStringStatus(_(menu_iteminfos[mode].status_str));
+      } else if (input->xbutton.button == Button2 && mode >= MAXCHOICES) {
+         return MainMenu();
       }
    }
    return INVALID;
