@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/choose.c,v 4.33 2000/04/17 20:56:57 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/choose.c,v 4.36 2001/08/06 21:38:57 william Exp $
  */
 
 #define _INCLUDE_FROM_CHOOSE_C_
@@ -1782,8 +1782,8 @@ void ButtonPressInPath(button_ev)
 
    if (button_ev->button == Button1) {
    } else if (button_ev->button == Button2) {
-      int buf_len=0;
-      char *cut_buffer;
+      int buf_len=0, from_selection=FALSE;
+      char *cut_buffer=NULL;
 
       switch (pni->edit_style) {
       case NAMES_SIMPLE_SELECT_NAME: return;
@@ -1794,7 +1794,7 @@ void ButtonPressInPath(button_ev)
       case NAMES_EDIT_ATTR: break;
       case NAMES_EDIT_NAME: break;
       }
-      cut_buffer = FetchCutBuffer(&buf_len);
+      cut_buffer = FetchSelectionOrCutBuffer(&buf_len, &from_selection);
       if (cut_buffer != NULL) {
          char *c_ptr=cut_buffer;
          int max_len;
@@ -1822,7 +1822,7 @@ void ButtonPressInPath(button_ev)
             RedrawNamePath();
             RedrawDspWindow();
          }
-         XFree(cut_buffer);
+         FreeSelectionOrCutBuffer(cut_buffer, from_selection);
       }
    } else if (button_ev->button == Button3) {
    }
