@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cmd.c,v 4.39 2000/03/08 02:22:31 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cmd.c,v 4.43 2001/10/08 22:35:53 william Exp $
  */
 
 #define _INCLUDE_FROM_CMD_C_
@@ -488,7 +488,8 @@ void UndoDeleteCmd(CmdPtr)
    sel_ptr = topSel;
 
    if (CmdPtr->pos_before_has_ids) {
-      pos_to_match = GetPositionFromFullID((char*)(CmdPtr->pos_before[count]));
+      pos_to_match = GetPositionFromFullID(
+            (char*)(long)(CmdPtr->pos_before[count]));
    } else {
       pos_to_match = CmdPtr->pos_before[count];
    }
@@ -502,7 +503,7 @@ void UndoDeleteCmd(CmdPtr)
 
          if (CmdPtr->pos_before_has_ids) {
             pos_to_match = GetPositionFromFullID(
-                  (char*)(CmdPtr->pos_before[count]));
+                  (char*)(long)(CmdPtr->pos_before[count]));
          } else {
             pos_to_match = CmdPtr->pos_before[count];
          }
@@ -708,7 +709,8 @@ int RedoDeleteCmd(CmdPtr)
       if (CmdPtr->pos_before_has_ids) {
          for (obj_ptr=topObj; obj_ptr != NULL; obj_ptr=next_obj) {
             next_obj = obj_ptr->next;
-            if (ObjHasFullID(obj_ptr, (char*)(CmdPtr->pos_before[count]))) {
+            if (ObjHasFullID(obj_ptr,
+                  (char*)(long)(CmdPtr->pos_before[count]))) {
                count++;
                sel_ptr->obj = obj_ptr;
                UnlinkObj(obj_ptr);
@@ -773,7 +775,8 @@ int UndoOrRedoMoveCmd(CmdPtr)
       if (CmdPtr->pos_before_has_ids) {
          for (obj_ptr=topObj; obj_ptr != NULL; obj_ptr=next_obj) {
             next_obj = obj_ptr->next;
-            if (ObjHasFullID(obj_ptr, (char*)(CmdPtr->pos_before[count]))) {
+            if (ObjHasFullID(obj_ptr,
+                  (char*)(int)(CmdPtr->pos_before[count]))) {
                count++;
                sel_ptr->obj = obj_ptr;
                if (!obj_ptr->locked) MoveObj(obj_ptr, dx, dy);
@@ -1804,7 +1807,7 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
 
       if ((psz_wb_data=SerializeCmd(CmdType, SubCmdPtr, TopSel, BotSel,
             NumObjs, &logical_clock)) != NULL) {
-         if (!SendWBData(psz_wb_data)) {
+         if (!SendWBData(psz_wb_data,logical_clock)) {
             wb_send_data_failed = TRUE;
          }
          SerializeFreeBuf(psz_wb_data);
@@ -1927,7 +1930,7 @@ void AbortPrepareCmd(CmdType)
          int i=0;
 
          for (i=0; i < stackingCount; i++) {
-            UtilFree((char*)(stackingPosition[i]));
+            UtilFree((char*)(long)(stackingPosition[i]));
          }
       }
       free(stackingPosition);
