@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cmdids.h,v 4.55 2000/08/04 13:24:09 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cmdids.h,v 4.65 2001/10/13 21:22:06 william Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -388,7 +388,30 @@
 #define CMDID_RENAMESIGNAMEFORAPORT        435
 #define CMDID_CLEARSIGNAMEFORAPORT         436
 #define CMDID_TOGGLESHOWWIRESIGNALNAME     437
+#define CMDID_TOGGLESHOWCHOICEBAR          438
+#define CMDID_MERGEPORTSWITHANOBJECT       439
+#define CMDID_RENUMBER_OBJECT_IDS          440
+#define CMDID_REPEATCONNECT2PORTSBYWIRE    441
+#define CMDID_CONNECTPORTSTOBROADCAST      442
+#define CMDID_IMPORTMULTIPAGETEXTFILE      443
+#define CMDID_SETMARGINSONIMPORTTEXT       444
+#define CMDID_TOGGLEWORDWRAPONIMPORTTEXT   445
+#define CMDID_DATA_IN_MBUFF                446
+#define CMDID_BENCHMARK                    447
+#define CMDID_CONVERTTOBEZIER              448
+#define CMDID_SETBEZIERCONVERTNUMSEGS      449
+#define CMDID_ADDSQUARETICKMARKS           450
+#define CMDID_ADDTRIANGLETICKMARKS         451
+#define CMDID_ADDCIRCLETICKMARKS           452
+#define CMDID_ADDXTICKMARKS                453
+#define CMDID_ADDDIAMONDTICKMARKS          454
+#define CMDID_ADDBOWTIETICKMARKS           455
+#define CMDID_ADDINVTRIANGLETICKMARKS      456
+#define CMDID_ADDPLUSTICKMARKS             457
+#define CMDID_ADDHOURGLASSTICKMARKS        458
+#define CMDID_SETTICKMARKSIZE              459
+#define CMDID_PEEKDIMENSION                460
 
-#define MAXCMDIDS 438
+#define MAXCMDIDS 461
 
 #endif /*_CMDIDS_H_*/
