@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/dialog.c,v 4.20 2000/07/18 18:43:36 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/dialog.c,v 4.23 2001/08/06 21:39:00 william Exp $
  */
 
 #define _INCLUDE_FROM_DIALOG_C_
@@ -888,14 +888,13 @@ void HandlePasteInDialog(MBInfoPtr, input)
    XEvent *input;
 {
    XButtonEvent *button_ev=(&(input->xbutton));
-   int buf_len=0;
+   int buf_len=0, from_selection=FALSE;
    char *cut_buffer=NULL;
 
    if (button_ev->button != Button2) return;
 
-   cut_buffer = FetchCutBuffer(&buf_len);
+   cut_buffer = FetchSelectionOrCutBuffer(&buf_len, &from_selection);
    if (cut_buffer == NULL) return;
-
    if ((unsigned char)(*cut_buffer) != TGIF_HEADER &&
          MBInfoPtr->return_str != NULL) {
       unsigned char *c_ptr=(unsigned char *)cut_buffer;
@@ -919,7 +918,7 @@ void HandlePasteInDialog(MBInfoPtr, input)
                myFgPixel);
       }
    }
-   XFree(cut_buffer);
+   FreeSelectionOrCutBuffer(cut_buffer, from_selection);
 }
 
 static
