@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.39 2000/07/22 20:01:22 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.43 2001/08/06 21:39:04 william Exp $
  */
 
 #define _INCLUDE_FROM_GRID_C_
@@ -508,6 +508,20 @@ void ToggleShowStatus()
    UpdatePinnedMenu(MENU_LAYOUT);
 }
 
+void ToggleShowChoicebar()
+{
+   noChoiceWindow = !noChoiceWindow;
+   if (noChoiceWindow) {
+      XUnmapWindow(mainDisplay, choiceWindow);
+      XUnmapWindow(mainDisplay, msgWindow);
+   } else {
+      XMapWindow(mainDisplay, choiceWindow);
+      XMapWindow(mainDisplay, msgWindow);
+   }
+   Reconfigure(TRUE);
+   UpdatePinnedMenu(MENU_LAYOUT);
+}
+
 void ToggleShowMode()
 {
    noModeWindow = !noModeWindow;
@@ -1288,7 +1302,7 @@ TgMenu *CreateExportFormatMenu(parent_menu, x, y, menu_info, status_str_xlated)
          strcpy(gszMsgBox, ExportFormatMenuLoadString(i, colorDump));
       } else {
          sprintf(gszMsgBox, ExportFormatMenuLoadString(i, colorDump),
-               GetExportName(whereToPrint, EXPORT_THIS));
+               GetExportName(i, EXPORT_THIS));
       }
       item_info->status_str = UtilStrDup(gszMsgBox);
       if (item_info->status_str == NULL) FailAllocMessage();
@@ -1362,6 +1376,9 @@ int RefreshShowHideMenu(menu)
    /* ShowStatus */
    ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWSTATUS,
          !noStatusWindow);
+   /* ShowChoice */
+   ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWCHOICEBAR,
+         !noChoiceWindow);
 
    return ok;
 }
