@@ -3,9 +3,9 @@
  *              WWW/Library/Implementation/HTTP.c distributed with lynx-2.2,
  *              whose original author is Tim Berners-lee <timbl@info.cern.ch>.
  *
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -32,7 +32,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/http.c,v 4.18 2000/01/15 22:23:46 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/http.c,v 4.21 2001/08/06 21:39:04 william Exp $
  */
 
 #define _INCLUDE_FROM_HTTP_C_
@@ -1345,7 +1345,9 @@ int HttpDoRead(n_socket, ppsz_buf, pn_buf_sz)
                      content_len, (*progress_buf=='\0' ? "" : progress_buf));
             }
          }
-         if (chopped_len != (-1)) len = chopped_len;
+         if (chopped_len != (-1)) {
+            len = chopped_len;
+         }
       }
    } while (status == TG_REMOTE_STATUS_OK && !end_of_file);
 
