@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/import.c,v 4.61 2000/07/25 20:25:05 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/import.c,v 4.67 2001/08/11 22:04:02 william Exp $
  */
 
 #define _INCLUDE_FROM_IMPORT_C_
@@ -685,8 +685,9 @@ void ImportXPixmapFile()
    importingFile = FALSE;
 }
 
-void ImportEPSFile(Embed)
+void ImportEPSFile(Embed, psz_fname_spec)
    int Embed;
+   char *psz_fname_spec;
 {
    char eps_fname[MAXPATHLENGTH+1], *rest=NULL, tmp_fname[MAXPATHLENGTH+1];
    char **lines=NULL, write_date[32];
@@ -702,20 +703,42 @@ void ImportEPSFile(Embed)
 
    importingFile = TRUE;
    *eps_fname = *tmp_fname = '\0';
-   if (importFromLibrary) {
-      char name[MAXSTRING+1], path[MAXSTRING+1];
+   if (psz_fname_spec != NULL && strcmp(psz_fname_spec, "-1") != 0) {
+      int len=strlen(psz_fname_spec);
 
-      if (SelectFromLibrary(TgLoadString(STID_SEL_EPS_FILE_TO_IMPORT),
-            EPSF_FILE_EXT, name, path) == INVALID) {
+      if (len > 0 && psz_fname_spec[len-1] == ')') {
+         psz_fname_spec[len-1] = '\0';
+         UtilStrCpyN(tmp_fname, sizeof(tmp_fname), psz_fname_spec);
+         psz_fname_spec[len-1] = ')';
+      } else {
+         UtilStrCpyN(tmp_fname, sizeof(tmp_fname), psz_fname_spec);
+      }
+      if (*tmp_fname == DIR_SEP) {
+         strcpy(eps_fname, tmp_fname);
+      } else if (curDirIsLocal) {
+         sprintf(eps_fname, "%s%c%s", curDir, DIR_SEP, tmp_fname);
+      } else {
+         sprintf(eps_fname, "%s%c%s", curLocalDir, DIR_SEP, tmp_fname);
+      }
+      *tmp_fname = '\0';
+   } else {
+      if (importFromLibrary) {
+         char name[MAXSTRING+1], path[MAXSTRING+1];
+
+         if (SelectFromLibrary(TgLoadString(STID_SEL_EPS_FILE_TO_IMPORT),
+               EPSF_FILE_EXT, name, path) == INVALID) {
+            importingFile = FALSE;
+            return;
+         }
+         sprintf(eps_fname, "%s%c%s", path, DIR_SEP, name);
+      } else if (SelectFileNameToImport(
+            TgLoadString(STID_SEL_EPS_FILE_TO_IMPORT), EPSF_FILE_EXT,
+            eps_fname) == INVALID) {
          importingFile = FALSE;
          return;
       }
-      sprintf(eps_fname, "%s%c%s", path, DIR_SEP, name);
-   } else if (SelectFileNameToImport(TgLoadString(STID_SEL_EPS_FILE_TO_IMPORT),
-         EPSF_FILE_EXT, eps_fname) == INVALID) {
-      importingFile = FALSE;
-      return;
-   } else if (FileIsRemote(eps_fname)) {
+   }
+   if (FileIsRemote(eps_fname)) {
       int is_html=FALSE;
 
       if (!Embed) {
@@ -783,8 +806,11 @@ void ImportEPSFile(Embed)
    ShowCursor();
 
    AddObj(NULL, topObj, obj_ptr);
-   PlaceTopObj(obj_ptr);
-
+   if (psz_fname_spec == NULL) {
+      PlaceTopObj(obj_ptr);
+   } else {
+      MoveObj(obj_ptr, 0-obj_ptr->obbox.ltx, 0-obj_ptr->obbox.lty);
+   }
    SelectTopObj();
    RecordNewObjCmd();
    SetFileModified(TRUE);
@@ -1420,6 +1446,21 @@ int ConvertAnyToXpm(pii, pszAnyPath, pszXpmPath, xpm_path_sz)
    return TRUE;
 }
 
+static int gnEnableFailedImportMsgBox=TRUE;
+
+int GetEnableFailedImportMsgBox()
+{
+   return gnEnableFailedImportMsgBox;
+}
+
+int SetEnableFailedImportMsgBox(int enable)
+{
+   int saved_enable_failed_import_msgbox=gnEnableFailedImportMsgBox;
+
+   gnEnableFailedImportMsgBox = enable;
+   return saved_enable_failed_import_msgbox;
+}
+
 static
 int DoImportOtherFile(pii, psz_path)
    struct ImportInfoRec *pii;
@@ -1496,7 +1537,9 @@ int DoImportOtherFile(pii, psz_path)
    if (rc != BitmapSuccess) {
       sprintf(gszMsgBox, TgLoadString(STID_CANT_IMPORT_GIVEN_TYPE_FILE),
             pii->name, (short_name ? rest : szOtherPath));
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      if (gnEnableFailedImportMsgBox) {
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
       unlink(szXpmPath);
       if (remote_file) unlink(tmp_fname);
       importingFile = FALSE;
@@ -2359,7 +2402,8 @@ fprintf(stderr, "maxBrowseX = %1d\n", maxBrowseX);
             int skip=FALSE;
 
 #ifndef _NO_LSTAT
-            if (lstat(name, &stat_buf) == 0 && (stat_buf.st_mode & S_IFLNK)) {
+            if (lstat(name, &stat_buf) == 0 &&
+                  ((stat_buf.st_mode & S_IFLNK) == S_IFLNK)) {
                sprintf(gszMsgBox, TgLoadString(STID_SKIP_GIVEN_FILE_SYM_LINK),
                      &name[savedDirNameLen+1]);
                Msg(gszMsgBox);
