@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.69 2000/08/07 20:04:10 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.80 2001/10/14 18:02:05 william Exp $
  */
 
 #define _INCLUDE_FROM_MAINLOOP_C_
@@ -67,6 +67,7 @@
 #include "inmethod.e"
 #include "mainloop.e"
 #include "mainmenu.e"
+#include "mark.e"
 #include "markup.e"
 #include "menu.e"
 #include "miniline.e"
@@ -195,6 +196,26 @@ void CleanUpExtraWinInfo()
    extraWinInfo = NULL;
 }
 
+void CleanUpTalkToSelfFiledes()
+{
+   if (talkToSelfFiledes[0] != (-1)) close(talkToSelfFiledes[0]);
+   if (talkToSelfFiledes[1] != (-1)) close(talkToSelfFiledes[1]);
+
+   talkToSelfFiledes[0] = talkToSelfFiledes[1] = (-1);
+}
+
+int InitTalkToSelfFiledes()
+{
+   talkToSelfFiledes[0] = talkToSelfFiledes[1] = (-1);
+
+   if (pipe(talkToSelfFiledes) != 0) {
+      sprintf(gszMsgBox, TgLoadString(STID_CREATE_SELF_PIPE_FAIL), TOOL_NAME);
+      fprintf(stderr, "%s\n", gszMsgBox);
+      return FALSE;
+   }
+   return TRUE;
+}
+
 static
 char *MySetLocale(category, new_locale)
    int category;
@@ -304,10 +325,12 @@ void CleanUp()
 #endif /* _TANGRAM_II */
 
    CleanUpMiniLines();
+   CleanUpCutPaste();
    CleanUpProperties();
    CleanUpExportFilters();
    ResetPSInfo();
    CleanUpEdit();
+   CleanUpMark();
    CleanUpShape();
    CleanUpHelp();
    CleanUpImport();
@@ -349,6 +372,7 @@ void CleanUp()
    CleanUpRecentFiles();
    CleanUpMeasureTooltip();
    CleanUpSpecial();
+   CleanUpTalkToSelfFiledes();
 
    if (iconWindowCreated) {
       XDestroyWindow(mainDisplay, iconBaseWindow);
@@ -1209,22 +1233,77 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    return TRUE;
 }
 
+void HandleDataInMBuff()
+{
+#ifdef _TGIF_WB2
+   TryHandleWBInputData();
+#endif /* _TGIF_WB2 */
+}
+
 static
 int GetAnXEvent(pXEvent)
    XEvent *pXEvent;
 {
 #ifdef _TGIF_WB2
-   if (!XPending(mainDisplay)) {
-      TryHandleWBInputData();
-      return FALSE;
-   }
    gstWBInfo.BlockRemoteCmdDepth--;
    if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
          drawWindow)) {
       if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
       }
    } else {
-      XNextEvent(mainDisplay, pXEvent);
+      if (XPending(mainDisplay)) {
+         XNextEvent(mainDisplay, pXEvent);
+      } else {
+         struct timeval timeout;
+         fd_set fdset;
+         int select_width=XConnectionNumber(mainDisplay)+1, status=0, nfds=0;
+
+         nfds = max(select_width, (int)(talkToSelfFiledes[0]+1));
+
+         timeout.tv_sec = 15;
+         timeout.tv_usec = 0;
+
+         FD_ZERO(&fdset);
+         FD_SET(select_width-1, &fdset);
+         FD_SET(talkToSelfFiledes[0], &fdset);
+#ifdef __hpux
+         status = select(nfds, (int*)&fdset, NULL, NULL, &timeout);
+#else /* !__hpux */
+         status = select(nfds, &fdset, NULL, NULL, &timeout);
+#endif /* __hpux */
+         if (status < 0) {
+            sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+                  "WaitForEvent()"),
+            fprintf(stderr, "%s\n", gszMsgBox);
+            return FALSE;
+         } else if (status == 0) {
+            /* timeout */
+            return FALSE;
+         } else if (status == 1) {
+            if (FD_ISSET(talkToSelfFiledes[0], &fdset)) {
+               char buf[2];
+
+               if (read(talkToSelfFiledes[0], buf, 1) != 1) {
+                  sprintf(gszMsgBox,
+                        TgLoadString(STID_READ_FROM_SELF_PIPE_FAIL), 1,
+                        TOOL_NAME),
+                  fprintf(stderr, "%s\n", gszMsgBox);
+               } else {
+                  SendCommandToSelf(CMDID_DATA_IN_MBUFF, 0);
+               }
+               return FALSE;
+            } else if (FD_ISSET(select_width-1, &fdset)) {
+               XNextEvent(mainDisplay, pXEvent);
+            } else {
+               return FALSE;
+            }
+         } else {
+            sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
+                  "GetAnXEvent()", status);
+            fprintf(stderr, "%s\n", gszMsgBox);
+            return FALSE;
+         }
+      }
    }
    gstWBInfo.BlockRemoteCmdDepth++;
 #endif /* _TGIF_WB2 */
@@ -1323,7 +1402,9 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
    } else if (statusWindow != None && input.xany.window == statusWindow) {
       StatusEventHandler(&input);
    } else if (modeWindow != None && input.xany.window == modeWindow) {
-      ModeEventHandler(&input);
+      if ((rc=ModeEventHandler(&input)) != INVALID) {
+         return rc;
+      }
    } else if (input.xany.window == dummyWindow1 ||
          input.xany.window == dummyWindow2) {
       DummiesEventHandler(&input);
@@ -1416,23 +1497,8 @@ void MainLoop(Op, FileName, FuncStr, Str1, Menu1, Str2, Menu2, Str3, Menu3)
    if (strcmp(Op, "init") == 0 || strcmp(Op, "justinit") == 0) {
       char szMsg1[80], szMsg2[80];
 
-      if (TGIF_PATCHLEVEL == 0) {
-         if (*specialBuild == '\0') {
-            sprintf(szMsg1, "%s Version %s", TOOL_NAME, versionString);
-         } else {
-            sprintf(szMsg1, "%s Version %s (%s)", TOOL_NAME, versionString,
-                  specialBuild);
-         }
-      } else {
-         if (*specialBuild == '\0') {
-            sprintf(szMsg1, "%s Version %s (patchlevel %1d)", TOOL_NAME,
-                  versionString, TGIF_PATCHLEVEL);
-         } else {
-            sprintf(szMsg1, "%s Version %s (patchlevel %1d - %s)", TOOL_NAME,
-                  versionString, TGIF_PATCHLEVEL, specialBuild);
-         }
-      }
-      sprintf(szMsg2, "%s %s", copyrightString, "(william@cs.UCLA.edu)");
+      GetTgifVersionAndPatchLevel(szMsg1, sizeof(szMsg1));
+      sprintf(szMsg2, "%s (%s)", copyrightString, authorEmailString);
       TwoLineMsg(szMsg1, szMsg2);
       if (!cmdLineDumpURL) {
          fprintf(stderr, "%s\n", szMsg1);
