@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.97 2000/08/04 13:39:18 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.109 2001/10/15 15:23:56 william Exp $
  */
 
 #define _INCLUDE_FROM_MENUINFO_C_
@@ -38,6 +38,7 @@
 
 #include "align.e"
 #include "color.e"
+#include "cutpaste.e"
 #include "drawing.e"
 #include "edit.e"
 #include "file.e"
@@ -483,6 +484,21 @@ static TgMenuItemInfo browseMenuItemInfo[] = {
 static TgMenuInfo browseMenuInfo={ TGMUTYPE_TEXT, browseMenuItemInfo,
                                    TgCreateMenuFromMenuInfo };
 
+static TgMenuItemInfo importMultipageTextMenuItemInfo[] = {
+   { N_("Import Multipage Text File"), NULL, N_("Import a multipage text file"),
+     NULL, CMDID_IMPORTMULTIPAGETEXTFILE },
+   { N_("Set Margins"), NULL,
+     N_("Set margins for importing multipage text files"), NULL,
+     CMDID_SETMARGINSONIMPORTTEXT },
+   { N_("Word Wrap"), NULL,
+     N_("Turn word wrapping on and off for importing multipage text files"),
+     NULL, CMDID_TOGGLEWORDWRAPONIMPORTTEXT },
+   { NULL, NULL, NULL, NULL, INVALID }
+};
+static TgMenuInfo importMultipageTextMenuInfo={ TGMUTYPE_TEXT,
+		importMultipageTextMenuItemInfo,
+		CreateImportMutipageTextFileMenu };
+
 static TgMenuItemInfo captureMenuItemInfo[] = {
    { N_("Screen Capture"), "Cntrl+1",
      N_("Capture a rectangular area of the screen"), NULL,
@@ -526,6 +542,13 @@ static TgMenuItemInfo importMenuItemInfo[] = {
      N_("Auto-generate a preview bitmap for a PS/EPS file when embedding/importing it (if it does not have one already)."), NULL,
      CMDID_TOGGLEAUTOEPSPREVIEWBITMAP },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Paste From File..."), NULL,
+     N_("Select a file and paste the content as text"), NULL,
+     CMDID_PASTEFROMFILE },
+   { N_("Import Multipage Text File"), TGMUITEM_SUBMENU,
+     N_("Import Multipage Text File submenu >>>"),
+     &importMultipageTextMenuInfo, INVALID },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Browse"), TGMUITEM_SUBMENU,
      N_("Browse submenu >>>"), &browseMenuInfo, INVALID },
    { N_("Screen Capture"), TGMUITEM_SUBMENU,
@@ -704,6 +727,44 @@ static TgMenuItemInfo bitPixmapMenuItemInfo[] = {
 static TgMenuInfo bitPixmapMenuInfo={ TGMUTYPE_TEXT, bitPixmapMenuItemInfo,
                                       CreateBitPixmapMenu };
 
+static TgMenuItemInfo addTickMarksMenuItemInfo[] = {
+   { N_("Set Tick Mark Size..."), NULL,
+     N_("Set the size of tick marks for various commands"), NULL,
+     CMDID_SETTICKMARKSIZE },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Add Squares"), NULL,
+     N_("Add square tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDSQUARETICKMARKS },
+   { N_("Add Triangles"), NULL,
+     N_("Add triangle tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDTRIANGLETICKMARKS },
+   { N_("Add Circles"), NULL,
+     N_("Add circle tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDCIRCLETICKMARKS },
+   { N_("Add Xs"), NULL,
+     N_("Add X tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDXTICKMARKS },
+   { N_("Add Diamonds"), NULL,
+     N_("Add diamond tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDDIAMONDTICKMARKS },
+   { N_("Add Bowties"), NULL,
+     N_("Add bowtie tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDBOWTIETICKMARKS },
+   { N_("Add Inverted Triangles"), NULL,
+     N_("Add inverted triangle tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDINVTRIANGLETICKMARKS },
+   { N_("Add Plus"), NULL,
+     N_("Add plus tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDPLUSTICKMARKS },
+   { N_("Add Hour Glasss"), NULL,
+     N_("Add hour glass tick marks at vertices of selected polys/polygons/splines"),
+     NULL, CMDID_ADDHOURGLASSTICKMARKS },
+   { NULL, NULL, NULL, NULL, INVALID }
+};
+static TgMenuInfo addTickMarksMenuInfo={ TGMUTYPE_TEXT,
+                                         addTickMarksMenuItemInfo,
+                                         TgCreateMenuFromMenuInfo };
+
 static TgMenuItemInfo polyMenuItemInfo[] = {
    { N_("Delete Point"), "Alt+Cntrl+d",
      N_("Delete vertices from a poly/polygon/spline"), NULL,
@@ -718,6 +779,13 @@ static TgMenuItemInfo polyMenuItemInfo[] = {
      N_("Toggle smooth/hinge vertices for a selected poly/polygon/spline"),
      NULL, CMDID_TOGGLESMOOTHHINGE },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Convert To Bezier Curve"), NULL,
+     N_("Convert selected splines to Bezier curves"), NULL,
+     CMDID_CONVERTTOBEZIER },
+   { N_("Set Segments For Convert To Bezier..."), NULL,
+     N_("Set the number of segments for the convert to Bezier command"), NULL,
+     CMDID_SETBEZIERCONVERTNUMSEGS },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Join Poly"), NULL, N_("Join polylines or open-splines at endpoints"),
      NULL, CMDID_JOINPOLY },
    { N_("Cut Poly"), NULL, N_("Cut a poly/polygon at a vertex into two pieces"),
@@ -726,6 +794,9 @@ static TgMenuItemInfo polyMenuItemInfo[] = {
    { N_("Make Regular Polygon..."), "Alt+\"",
      N_("Make a selected polygon/closed spline into a regular one"), NULL,
      CMDID_MAKEREGPOLYGON },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Add Tick Marks"), TGMUITEM_SUBMENU,
+     N_("Add Tick Marks submenu >>>"), &addTickMarksMenuInfo, INVALID },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo polyMenuInfo={ TGMUTYPE_TEXT, polyMenuItemInfo,
@@ -769,7 +840,7 @@ static TgMenuItemInfo textMenuItemInfo[] = {
      N_("Remove any vertical offset at the text cursor"), NULL,
      CMDID_REMOVEVERTICALOFFSET },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
-   { N_("Set Script Fraction"), NULL,
+   { N_("Set Script Fraction..."), NULL,
      N_("Set the relative size of super/subscript text"), NULL,
      CMDID_SETSCRIPTFRACTION },
    { NULL, NULL, NULL, NULL, INVALID }
@@ -785,6 +856,9 @@ static TgMenuItemInfo createObjectMenuItemInfo[] = {
      N_("Read points from terminal and create a polygon"),
      NULL, CMDID_INPUTPOLYGONPTS },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Add Tick Marks"), TGMUITEM_SUBMENU,
+     N_("Add Tick Marks submenu >>>"), &addTickMarksMenuInfo, INVALID },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Specify An Arc..."), "Alt+9", N_("Create and specify an arc"), NULL,
      CMDID_SPECIFYANARC },
    { N_("Get Bounding Box"), NULL,
@@ -1047,6 +1121,8 @@ static TgMenuItemInfo showHideMenuItemInfo[] = {
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Show Menubar"), NULL, N_("Show menubar window"), NULL,
      CMDID_TOGGLESHOWMENUBAR },
+   { N_("Show Choice"), NULL, N_("Show choice and message window"), NULL,
+     CMDID_TOGGLESHOWCHOICEBAR },
    { N_("Show Status"), NULL, N_("Show status window"), NULL,
      CMDID_TOGGLESHOWSTATUS },
    { NULL, NULL, NULL, NULL, INVALID }
@@ -1329,6 +1405,10 @@ static TgMenuItemInfo animateMenuItemInfo[] = {
    { N_("Animate Flash"), "Alt+f",
      N_("Flash a selected poly (not very useful)"), NULL,
      CMDID_ANIMATEFLASH },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Renumber Object IDs"), "Alt+f",
+     N_("Renumber object IDs for all objects"), NULL,
+     CMDID_RENUMBER_OBJECT_IDS },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo animateMenuInfo={ TGMUTYPE_TEXT, animateMenuItemInfo,
@@ -1338,10 +1418,21 @@ static TgMenuItemInfo portMenuItemInfo[] = {
    { N_("Connect Two Ports by a Wire..."), NULL,
      N_("Connect two ports by a wire"), NULL,
      CMDID_CONNECT2PORTSBYWIRE },
+   { N_("Repeat Connect Two Ports by a Wire..."), NULL,
+     N_("Repeatedly connecting two ports by a wire (will not prompt if the wire name can be detected).  Wire signal name will not be shown."), NULL,
+     CMDID_REPEATCONNECT2PORTSBYWIRE },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Show Wire Signal Name"), NULL,
      N_("Toggle between showing (with placing) and hiding the wire signal name when connecting ports"), NULL,
      CMDID_TOGGLESHOWWIRESIGNALNAME },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Merge Ports with An Object"), NULL,
+     N_("Merge selected floating port objects with a selected composite object"), NULL,
+     CMDID_MERGEPORTSWITHANOBJECT },
+   { N_("Connect Ports to a Broadcast Wire"), NULL,
+     N_("Connect selected floating port objects to a selected broadcast-type wire"), NULL,
+     CMDID_CONNECTPORTSTOBROADCAST },
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Clear Signal Name for a Port..."), NULL,
      N_("Clear the 'signal_name=' attribute for a port"), NULL,
      CMDID_CLEARSIGNAMEFORAPORT },
@@ -1448,6 +1539,9 @@ TgMenuInfo editAttrInEditorMenuInfo={ TGMUTYPE_TEXT, NULL,
 
 TgMenuInfo getPropertyMenuInfo={ TGMUTYPE_TEXT, NULL, CreateGetPropertyMenu };
 
+TgMenuInfo peekDimensionMenuInfo={ TGMUTYPE_TEXT, NULL,
+                                   CreatePeekDimensionMenu };
+
 static TgMenuItemInfo baseContextMenuItemInfo[] = {
    { N_("Mode"), TGMUITEM_PINNABLESUBMENU, N_("Mode Menu"), &modeMenuInfo,
      MENU_MODE },
@@ -1469,6 +1563,8 @@ static TgMenuItemInfo baseContextMenuItemInfo[] = {
      INVALID },
    { N_("Get Property"), TGMUITEM_SUBMENU,
      N_("Get property submenu >>>"), &getPropertyMenuInfo, INVALID },
+   { N_("Peek Dimension"), TGMUITEM_SUBMENU,
+     N_("Peek dimension submenu >>>"), &peekDimensionMenuInfo, INVALID },
    { NULL, NULL, NULL, NULL, INVALID }
 };
 TgMenuInfo baseContextMenuInfo={ TGMUTYPE_TEXT, baseContextMenuItemInfo,
