@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.47 2000/10/29 15:45:28 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.57 2001/10/15 20:54:10 william Exp $
  */
 
 #define _INCLUDE_FROM_MSG_C_
@@ -1337,27 +1337,39 @@ void Error(where, msg)
    exit(-1);
 }
 
-void GetTgifVersionAndPatchLevel(buf)
+void GetTgifVersionAndPatchLevel(buf, buf_sz)
    char *buf;
+   int buf_sz;
 {
+   char buf1[MAXSTRING], buf2[MAXSTRING];
+
+   *buf1 = *buf2 = '\0';
    if (TGIF_PATCHLEVEL == 0) {
-      /* do not translate -- program constants */
       if (*specialBuild == '\0') {
-         sprintf(buf, "%s Version %s\n", TOOL_NAME, versionString);
+         sprintf(buf1, TgLoadString(STID_TOOL_VERSION),
+               TOOL_NAME, versionString);
       } else {
-         sprintf(buf, "%s Version %s (%s)\n", TOOL_NAME, versionString,
-               specialBuild);
+         sprintf(buf1, TgLoadString(STID_TOOL_VERSION_SPC_BUILD),
+               TOOL_NAME, versionString, specialBuild);
       }
+      strcpy(buf2, versionString);
    } else {
-      /* do not translate -- program constants */
       if (*specialBuild == '\0') {
-         sprintf(buf, "%s Version %s (patchlevel %1d)\n", TOOL_NAME,
-               versionString, TGIF_PATCHLEVEL);
+         sprintf(buf1, TgLoadString(STID_TOOL_VERSION_PATCH),
+               TOOL_NAME, versionString, TGIF_PATCHLEVEL);
       } else {
-         sprintf(buf, "%s Version %s (patchlevel %1d - %s)\n", TOOL_NAME,
-               versionString, TGIF_PATCHLEVEL, specialBuild);
+         sprintf(buf1, TgLoadString(STID_TOOL_VERSION_PATCH_SPC_BUILD),
+               TOOL_NAME, versionString, TGIF_PATCHLEVEL, specialBuild);
       }
+      sprintf(buf2, "%s.%1d", versionString, TGIF_PATCHLEVEL);
    }
+#ifdef TGIF_DISTR_VER
+   if (strcmp(TGIF_DISTR_VER, buf2) != 0) {
+      sprintf(&buf1[strlen(buf1)], TgLoadString(STID_TGIF_DISTRUBITION),
+            TGIF_DISTR_VER);
+   }
+#endif /* TGIF_DISTR_VER */
+   UtilStrCpyN(buf, buf_sz, buf1);
 }
 
 void Usage(tool_name)
@@ -1365,7 +1377,7 @@ void Usage(tool_name)
 {
    char buf[MAXSTRING];
 
-   GetTgifVersionAndPatchLevel(buf);
+   GetTgifVersionAndPatchLevel(buf, sizeof(buf));
    fprintf(stderr, "%s\n", buf);
 
    fprintf(stderr, "%s\n\n", copyrightString);
@@ -1384,6 +1396,7 @@ void Usage(tool_name)
    fprintf(stderr, "\t[-exec <file>] \\\n");
    fprintf(stderr, "\t[-dbim {xcin|chinput|xim|kinput2}] \\\n");
    fprintf(stderr, "\t[-usexlib] \\\n");
+   fprintf(stderr, "\t[-listdontreencode] \\\n");
    fprintf(stderr, "\t[{<file>[.%s] | -merge <file1>[.%s] <file2>[.%s] ...}]\n",
          OBJ_FILE_EXT, OBJ_FILE_EXT, OBJ_FILE_EXT);
    fprintf(stderr, "or:\n");
@@ -1400,7 +1413,9 @@ void Usage(tool_name)
    fprintf(stderr, "\t[-gray] \\\n");
    fprintf(stderr, "\t[-color | -reqcolor] \\\n");
    fprintf(stderr, "\t[-adobe | -adobe=<number>/<number>] \\\n");
-   fprintf(stderr, "\t[-dontreencode=\"<fontfamily>:<fontfamily>:...\"] \\\n");
+   fprintf(stderr,
+         "\t[-dontreencode=\"<%s>:<%s>:...\" | -listdontreencode] \\\n",
+         "fontfamily", "fontfamily");
    fprintf(stderr, "\t[-producedby=\"<string>\"] \\\n");
    fprintf(stderr, "\t[-page <number>] \\\n");
    fprintf(stderr, "\t[-print_cmd \"<command>\"] \\\n");
@@ -1409,6 +1424,7 @@ void Usage(tool_name)
    fprintf(stderr, "\t[-nolandpdfspd] \\\n");
    fprintf(stderr, "\t[-dontcondense | -condensed] \\\n");
    fprintf(stderr, "\t[-a4] \\\n");
+   fprintf(stderr, "\t[-quiet] \\\n");
    fprintf(stderr, "\t[-bop_hook \"<string>\"] \\\n");
    fprintf(stderr, "\t[-eop_hook \"<string>\"] \\\n");
    fprintf(stderr, "\t[-tmp_file_mode \"<octal number>\"] \\\n");
@@ -1442,6 +1458,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             show_copyright = FALSE;
          } else if (strcmp(*argv, "-parse_html") == 0) {
             show_copyright = FALSE;
+         } else if (strcmp(*argv, "-quiet") == 0) {
+            show_copyright = FALSE;
          }
       }
    }
@@ -1450,10 +1468,9 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
    if (show_copyright) {
       char buf[MAXSTRING];
 
-      GetTgifVersionAndPatchLevel(buf);
-      fprintf(stderr, "%s\n", buf);
-
-      fprintf(stderr, "%s\n\n", copyrightString);
+      GetTgifVersionAndPatchLevel(buf, sizeof(buf));
+      fprintf(stderr, "%s", buf);
+      fprintf(stderr, "%s (%s)\n\n", copyrightString, authorEmailString);
       fflush(stderr);
    }
    if (!from_prtgif) {
@@ -1606,6 +1623,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             SetCmdLineCondensedPS();
          } else if (strcmp(*argv, "-a4") == 0) {
             cmdLineA4 = TRUE;
+         } else if (strcmp(*argv, "-quiet") == 0) {
+            cmdLineQuiet = TRUE;
          } else if (strcmp(*argv, "-bop_hook") == 0) {
             argc--; argv++;
             if (argc <= 0) {
@@ -1673,6 +1692,10 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
                usePsAdobeString = FALSE;
                *adobeString = *epsfString = '\0';
             }
+         } else if (strcmp(*argv, "-listdontreencode") == 0) {
+#ifdef _DONT_REENCODE
+            fprintf(stderr, "-D_DONT_REENCODE=\"%s\"\n", _DONT_REENCODE);
+#endif /* _DONT_REENCODE */
          } else if (strncmp(*argv, "-dontreencode=", 14) == 0) {
             strcpy(cmdLineDontReencode, &(*argv)[14]);
             UtilTrimBlanks(cmdLineDontReencode);
@@ -1877,6 +1900,10 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
          }
          strcat(geometrySpec, argv[i]);
          geometrySpecified = TRUE;
+      } else if (strcmp(argv[i], "-listdontreencode") == 0) {
+#ifdef _DONT_REENCODE
+         fprintf(stderr, "-D_DONT_REENCODE=\"%s\"\n", _DONT_REENCODE);
+#endif /* _DONT_REENCODE */
       } else if (*argv[i] == '=') {
          strcpy(geometrySpec, argv[i]);
          geometrySpecified = TRUE;
