@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/names.c,v 4.46 2000/10/25 19:05:09 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/names.c,v 4.48 2001/08/06 21:39:08 william Exp $
  */
 
 #define _INCLUDE_FROM_NAMES_C_
@@ -426,7 +426,7 @@ DspList *DirListing(Path, ExtStr, OtherExtStr)
                int show_msg=TRUE;
 #ifndef _NO_LSTAT
                if (lstat(s, &stat_buf) == 0 &&
-                     (stat_buf.st_mode & S_IFLNK)) {
+                     ((stat_buf.st_mode & S_IFLNK) == S_IFLNK)) {
                   show_msg = FALSE;
                }
 #endif /* _NO_LSTAT */
@@ -474,7 +474,7 @@ DspList *DirListing(Path, ExtStr, OtherExtStr)
             int show_msg=TRUE;
 #ifndef _NO_LSTAT
             if (lstat(s, &stat_buf) == 0 &&
-                  (stat_buf.st_mode & S_IFLNK)) {
+                  ((stat_buf.st_mode & S_IFLNK) == S_IFLNK)) {
                show_msg = FALSE;
             }
 #endif /* _NO_LSTAT */
