@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.35 2000/10/25 19:04:43 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.37 2001/09/04 14:36:37 william Exp $
  */
 
 #define _INCLUDE_FROM_NAVIGATE_C_
@@ -294,7 +294,7 @@ void CleanUpNavigate()
 
 void InitNavigate()
 {
-   char *c_ptr=NULL;
+   char *c_ptr=NULL, spec[MAXSTRING];
 
    visibleGridInSlideShow = FALSE;
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
@@ -318,6 +318,36 @@ void InitNavigate()
          lineWidthIndexInSlideShow = 4;
       }
    }
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"SlideShowWindowOffsets")) !=
+         NULL) {
+      UtilStrCpyN(spec, sizeof(spec), c_ptr);
+      UtilTrimBlanks(spec);
+      if (ParseXYSpec(spec, &slideShowXOffset, &slideShowYOffset)) {
+         slideShowInfoValid = TRUE;
+      } else {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "SlideShowWindowOffsets", spec, "0,0");
+         fprintf(stderr, "\n");
+      }
+   }
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"SlideShowBorderColor")) !=
+         NULL) {
+      int new_alloc=0;
+
+      UtilStrCpyN(spec, sizeof(spec), c_ptr);
+      UtilTrimBlanks(spec);
+
+      if (QuickFindColorIndex(NULL, spec, &new_alloc, FALSE) == INVALID) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_COLORXPM_GET),
+               TOOL_NAME, "SlideShowBorderColor", spec);
+         fprintf(stderr, "\n");
+         return;
+      }
+      if (slideShowBorderColor != NULL) UtilFree(slideShowBorderColor);
+      slideShowBorderColor = UtilStrDup(spec);
+      if (slideShowBorderColor == NULL) FailAllocMessage();
+      slideShowInfoValid = TRUE;
+   }
 }
 
 static
