@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/page.c,v 4.51 2000/07/22 20:17:31 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/page.c,v 4.54 2001/08/11 22:02:36 william Exp $
  */
 
 #define _INCLUDE_FROM_PAGE_C_
@@ -610,12 +610,10 @@ int EditOrSelectPageNames(TopStr, Which, entries, num_entries)
    return rc;
 }
 
-void GotoPage()
+void GotoPage(psz_page_num)
+   char *psz_page_num;
 {
-   int num_entries=0;
-   DspList *dsp_ptr;
-   char **entries=NULL;
-   int index;
+   int index=(-1);
 
    if (pageLayoutMode == PAGE_TILE) {
       MsgBox(TgLoadString(STID_CANT_DO_GOTOPAGE_IN_TILED), TOOL_NAME, INFO_MB);
@@ -624,28 +622,36 @@ void GotoPage()
    curChoiceBeforePageChange = curChoice;
    MakeQuiescent();
 
-   dsp_ptr = PageNameListing(&num_entries);
-   ignoreDirectoryFlag = TRUE;
-   entries = MakeNameDspItemArray(num_entries, dsp_ptr);
-   ignoreDirectoryFlag = FALSE;
+   if (psz_page_num != NULL && strcmp(psz_page_num, "-1") != 0) {
+      index = atoi(psz_page_num);
+   } else {
+      int num_entries=0;
+      DspList *dsp_ptr;
+      char **entries=NULL;
 
-   index = EditOrSelectPageNames(TgLoadString(STID_GOTO_PAGE_DOTS), PAGE_GOTO,
-         entries, num_entries);
+      dsp_ptr = PageNameListing(&num_entries);
+      ignoreDirectoryFlag = TRUE;
+      entries = MakeNameDspItemArray(num_entries, dsp_ptr);
+      ignoreDirectoryFlag = FALSE;
 
-   free(dsp_ptr);
-   free(*entries);
-   free(entries);
+      index = EditOrSelectPageNames(TgLoadString(STID_GOTO_PAGE_DOTS),
+            PAGE_GOTO, entries, num_entries);
 
-   if (index++ == (-1)) {
-      Msg(TgLoadString(STID_NO_PAGE_SELECTED));
-      return;
+      free(dsp_ptr);
+      free(*entries);
+      free(entries);
+
+      if (index++ == (-1)) {
+         Msg(TgLoadString(STID_NO_PAGE_SELECTED));
+         return;
+      }
    }
    if (index < 1 || index > lastPageNum) {
       sprintf(gszMsgBox, TgLoadString(STID_INVALID_GIVEN_PAGE_NUMBER), index);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return;
    }
-   if (index == curPageNum) {
+   if (psz_page_num == NULL && index == curPageNum) {
       sprintf(gszMsgBox, TgLoadString(STID_ALREADY_AT_GIVEN_PAGE), curPageNum);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return;
