@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/pin.c,v 4.13 2000/07/26 12:56:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/pin.c,v 4.15 2001/08/06 21:39:10 william Exp $
  */
 
 #define _INCLUDE_FROM_PIN_C_
@@ -180,11 +180,11 @@ int AddObjToSorted(pObj, ppTopSel, ppBotSel)
    struct ObjRec *pObj;
    struct SelRec **ppTopSel, **ppBotSel;
 {
-   int nIndex=(int)(pObj->userdata);
+   int nIndex=(int)(long)(pObj->userdata);
    struct SelRec *pSel=NULL;
 
    for (pSel=(*ppTopSel); pSel != NULL; pSel=pSel->next) {
-      if (nIndex > (int)(pSel->obj->userdata)) {
+      if (nIndex > (int)(long)(pSel->obj->userdata)) {
          return (AddObjIntoSel(pObj, pSel->prev, pSel, ppTopSel, ppBotSel) !=
                NULL);
       }
@@ -250,7 +250,7 @@ int FindExtraPoly(pTopSel, pBotSel, ppTopSel, ppBotSel, pnCount, pBBox)
 
             for (pObj=topObj; pObj != NULL; pObj=pObj->next, nIndex++) {
                if (pObj == pPolyObj) {
-                  pPolyObj->userdata = (void*)nIndex;
+                  pPolyObj->userdata = (void*)(long)nIndex;
                   if (AddObjToSorted(pPolyObj,
                         &pTopSortedSel, &pBotSortedSel)) {
                   }
