@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/remote.c,v 4.35 2000/10/25 19:04:27 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/remote.c,v 4.37 2001/08/06 21:39:12 william Exp $
  */
 
 #define _INCLUDE_FROM_REMOTE_C_
@@ -564,6 +564,7 @@ char *WriteRemoteFileIntoTemp(psz_buf, buf_sz, psz_ext)
       sprintf(gszMsgBox, TgLoadString(STID_CANT_CREATE_NAMED_TMP_FILE),
             tmp_fname);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      free(tmp_fname);
       return NULL;
    } else {
       int ok=TRUE;
@@ -577,6 +578,7 @@ char *WriteRemoteFileIntoTemp(psz_buf, buf_sz, psz_ext)
       close(fd);
       if (!ok) {
          unlink(tmp_fname);
+         free(tmp_fname);
          return NULL;
       }
       if (tmpFileMode != 0 && chmod(tmp_fname, tmpFileMode)) {
