@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.37 2000/08/03 18:05:18 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/select.c,v 4.39 2001/08/06 21:39:12 william Exp $
  */
 
 #define _INCLUDE_FROM_SELECT_C_
@@ -457,6 +457,23 @@ struct ObjRec *FindAVertex(XOff, YOff, VIndex, AbsX, AbsY)
    return NULL;
 }
 
+static
+int CandidatePortOwner(obj_ptr)
+   struct ObjRec *obj_ptr;
+{
+   if (obj_ptr->type == OBJ_SYM || obj_ptr->type == OBJ_ICON) {
+      return TRUE;
+   } else if (obj_ptr->type == OBJ_GROUP) {
+      struct AttrRec *attr_ptr=FindAttrWithName(obj_ptr, "type=", NULL);
+
+      if (attr_ptr != NULL && strcmp(attr_ptr->attr_value.s,
+            "tgBroadcastWire") == 0) {
+         return TRUE;
+      }
+   }
+   return FALSE;
+}
+
 struct ObjRec *FindAnObj(XOff, YOff, OwnerObj, ConnectObj, ReturnedObjName)
    int XOff, YOff;
    struct ObjRec **OwnerObj, **ConnectObj;
@@ -605,7 +622,7 @@ struct ObjRec *FindAnObj(XOff, YOff, OwnerObj, ConnectObj, ReturnedObjName)
                int just_seen_sym_or_icon=FALSE;
 
                if (!see_sym_or_icon) {
-                  if (obj_ptr->type == OBJ_SYM || obj_ptr->type == OBJ_ICON) {
+                  if (CandidatePortOwner(obj_ptr)) {
                      see_sym_or_icon = TRUE;
                      just_seen_sym_or_icon = TRUE;
                   }
@@ -627,8 +644,7 @@ struct ObjRec *FindAnObj(XOff, YOff, OwnerObj, ConnectObj, ReturnedObjName)
                      strcpy(c_ptr, attr_ptr->attr_value.s);
                      c_ptr = &c_ptr[strlen(c_ptr)];
                   }
-                  if (!just_seen_sym_or_icon && (obj_ptr->type == OBJ_SYM ||
-                        obj_ptr->type == OBJ_ICON)) {
+                  if (!just_seen_sym_or_icon && CandidatePortOwner(obj_ptr)) {
                      obj_ptr->tmp_child = NULL;
                      prev_obj = obj_ptr;
                      break;
