@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.60 2000/10/30 15:09:00 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.70 2001/10/14 00:19:58 william Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -257,6 +257,7 @@ Window	statusSubWindow[MAX_STATUS_BTNS];
 
 int	noMenubar=FALSE;
 int	noStatusWindow=FALSE;
+int	noChoiceWindow=FALSE;
 int	noModeWindow=FALSE;
 int	noChatWindow=TRUE;
 
@@ -266,6 +267,8 @@ int	curChoiceBeforeMakeQuiescent=NOTHING;
 
 int	noMinWinSize=TRUE;
 
+int	talkToSelfFiledes[2];
+
 static Atom	protocolAtom=(Atom)0;
 
 static int	canvasWindowOnly=FALSE;
@@ -385,8 +388,9 @@ void InitWinSizes()
    mainWinW = titleWindowW+(brdrW<<1);
    mainWinH = (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) +
          (noStatusWindow ? 0 : statusWindowH+(brdrW<<1)) +
+         (noChoiceWindow ? 0 : msgWindowH+(brdrW<<1)) +
          (noChatWindow ? 0 : chatWindowH+(brdrW<<1)) +
-         titleWindowH+choiceWindowH+vSBarH+scrollBarW+8*brdrW;
+         titleWindowH+vSBarH+scrollBarW+6*brdrW;
    if (colorLayers) vSBarH = DEF_VSBAR_H;
    CalcStatusSubWinGeom();
 }
@@ -399,9 +403,10 @@ void InverseInitWinSizes()
    statusWindowW = titleWindowW-userRedrawWindowW-(brdrW<<1);
    CalcMenubarWindowHeight();
    msgWindowW = titleWindowW-choiceWindowW-(brdrW<<1);
-   vSBarH = mainWinH - titleWindowH - choiceWindowH - scrollBarW - 8*brdrW -
+   vSBarH = mainWinH - titleWindowH - scrollBarW - 6*brdrW -
          (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) -
          (noStatusWindow ? 0 : statusWindowH+(brdrW<<1)) -
+         (noChoiceWindow ? 0 : msgWindowH+(brdrW<<1)) -
          (noChatWindow ? 0 : chatWindowH+(brdrW<<1));
    drawWinH = initDrawWinH = vSBarH-rulerW-(brdrW<<1);
    drawWinW = initDrawWinW = titleWindowW-scrollBarW-rulerW-(brdrW<<2) -
@@ -453,10 +458,12 @@ void DoReconfigure()
             menubarWindowH);
       cur_y += menubarWindowH + (brdrW<<1);
    }
-   XMoveWindow(mainDisplay, choiceWindow, 0, cur_y);
-   XMoveResizeWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1), cur_y,
-         msgWindowW, msgWindowH);
-   cur_y += msgWindowH + (brdrW<<1);
+   if (!noChoiceWindow) {
+      XMoveWindow(mainDisplay, choiceWindow, 0, cur_y);
+      XMoveResizeWindow(mainDisplay, msgWindow, choiceWindowW+(brdrW<<1), cur_y,
+            msgWindowW, msgWindowH);
+      cur_y += msgWindowH + (brdrW<<1);
+   }
    if (!noModeWindow) {
       XMoveResizeWindow(mainDisplay, modeWindow, 0, cur_y,
             modeWindowW, modeWindowH);
@@ -526,7 +533,8 @@ void DoReconfigure()
 
       XMoveResizeWindow(mainDisplay, drawWindow,
             new_x+draw_win_x_offset, new_y+draw_win_y_offset,
-            ZOOMED_SIZE(onePageWidth), ZOOMED_SIZE(onePageHeight));
+            ZOOMED_SIZE(onePageWidth)-(slideShowXOffset<<1),
+            ZOOMED_SIZE(onePageHeight-(slideShowYOffset<<1)));
       if (ZOOMED_SIZE(onePageWidth) > drawWinW) {
          drawWinW = ZOOMED_SIZE(onePageWidth);
          changed = TRUE;
@@ -590,8 +598,9 @@ void Reconfigure(Forced)
       initDrawWinH = mainWinH - titleWindowH -
             (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) -
             (noStatusWindow ? 0 : statusWindowH+(brdrW<<1)) -
+            (noChoiceWindow ? 0 : msgWindowH+(brdrW<<1)) -
             (noChatWindow ? 0 : chatWindowH+(brdrW<<1)) -
-            choiceWindowH - rulerW - scrollBarW - 10*brdrW;
+            rulerW - scrollBarW - 8*brdrW;
    }
    drawWinW = initDrawWinW;
    drawWinH = initDrawWinH;
@@ -677,6 +686,14 @@ int mainWinEventHandler(input)
    } else if (input->type == MapNotify) {
       UnIconify();
    } else if (input->type == ConfigureNotify) {
+#ifdef _DO_BENCHMARK_AT_START
+      static int first_time=TRUE;
+
+      if (first_time) {
+         SendCommandToSelf(CMDID_BENCHMARK, INVALID);
+         first_time = FALSE;
+      }
+#endif /* _DO_BENCHMARK_AT_START */
       configure = TRUE;
    } else if (input->type == VisibilityNotify &&
          input->xvisibility.state == VisibilityUnobscured) {
@@ -748,10 +765,26 @@ int mainWinEventHandler(input)
    return INVALID;
 }
 
+void CleanUpPaperSize()
+{
+   if (psPageHeightInInch != NULL) free(psPageHeightInInch);
+   if (psPageWidthInInch != NULL) free(psPageWidthInInch);
+   if (psYOff != NULL) free(psYOff);
+   if (psYOffStr != NULL) {
+      free(psYOffStr[0]);
+      free(psYOffStr[1]);
+      free(psYOffStr);
+   }
+   psYOffStr = NULL;
+   psYOff = psPageWidthInInch = psPageHeightInInch = NULL;
+}
+
 void InitPaperSize()
 {
    float fval=(float)0;
 
+   if (psYOffStr != NULL) CleanUpPaperSize();
+
    psYOffStr = (char**)malloc(MAXPAGESTYLES*sizeof(char *));
    if (psYOffStr == NULL) FailAllocMessage();
    psYOffStr[0] = (char*)malloc(20*sizeof(char));
@@ -774,20 +807,6 @@ void InitPaperSize()
    psPageHeightInInch[1] = ((float)onePageWidth)/((float)ONE_INCH);
 }
 
-void CleanUpPaperSize()
-{
-   if (psPageHeightInInch != NULL) free(psPageHeightInInch);
-   if (psPageWidthInInch != NULL) free(psPageWidthInInch);
-   if (psYOff != NULL) free(psYOff);
-   if (psYOffStr != NULL) {
-      free(psYOffStr[0]);
-      free(psYOffStr[1]);
-      free(psYOffStr);
-   }
-   psYOffStr = NULL;
-   psYOff = psPageWidthInInch = psPageHeightInInch = NULL;
-}
-
 static Window	savedMainWindow=None, savedDrawWindow=None;
 static Pixmap	wmIconPixmap=None, wmIconPixmapMask=None;
 
@@ -842,13 +861,65 @@ int GetIconPixmap(pPixmap, pPixmapMask)
    return TRUE;
 }
 
+void InitTmpDir()
+{
+   int tmp_dir_in_x_def=FALSE;
+   char *c_ptr=NULL;
+
+   if (!PRTGIF || cmdLineOpenDisplay) {
+      /* do not translate -- program constants */
+      if (mainDisplay != NULL &&
+            (c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "TmpDir")) != NULL) {
+         UtilTrimBlanks(c_ptr);
+         if (*c_ptr == '\0') {
+            fprintf(stderr, TgLoadString(STID_INVALID_DIR_IN_X_DEFAULT),
+                  c_ptr, TOOL_NAME, "TmpDir");
+            fprintf(stderr, "\n");
+         } else {
+            UtilStrCpyN(tmpDir, sizeof(tmpDir)-1, c_ptr);
+            strcat(tmpDir, DIR_SEP_STR);
+            if (UtilIsDirectory(tmpDir)) {
+               tmp_dir_in_x_def = TRUE;
+            } else {
+               fprintf(stderr, TgLoadString(STID_TMP_DIR_NOT_EXIST_USE_XDEF),
+                     tmpDir, TOOL_NAME, "TmpDir");
+               fprintf(stderr, "\n");
+            }
+         }
+      }
+   }
+   if (!tmp_dir_in_x_def) {
+#ifndef _TMP_DIR_IN_HOME_DIR
+      UtilStrCpyN(tmpDir, sizeof(tmpDir), TMP_DIR);
+      if (!PRTGIF || cmdLineOpenDisplay) {
+         if (mainDisplay != NULL &&
+               (c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "TmpDirInHomeDir")) !=
+               NULL && UtilStrICmp(c_ptr, "true") == 0) {
+            UtilStrCpyN(tmpDir, sizeof(tmpDir)-1, tgifDir);
+            strcat(tmpDir, DIR_SEP_STR);
+         }
+      }
+#else /* _TMP_DIR_IN_HOME_DIR */
+      UtilStrCpyN(tmpDir, sizeof(tmpDir)-1, tgifDir);
+      strcat(tmpDir, DIR_SEP_STR);
+      if (!PRTGIF || cmdLineOpenDisplay) {
+         if (mainDisplay != NULL &&
+               (c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "TmpDirInHomeDir")) !=
+               NULL && UtilStrICmp(c_ptr, "false") == 0) {
+            UtilStrCpyN(tmpDir, sizeof(tmpDir), TMP_DIR);
+         }
+      }
+#endif /* ~_TMP_DIR_IN_HOME_DIR */
+   }
+}
+
 void Setup()
 {
    int bitmask=0, x_neg_in_def=FALSE, y_neg_in_def=FALSE;
    int x_in_cmdline=FALSE, y_in_cmdline=FALSE, cur_y=0, i, x_offset=0;
    int bg_pixel=(-1);
    char *c_ptr=NULL;
-   int def_x_neg=0, def_y_neg=0, paper_size_set, tmp_dir_in_x_def=FALSE;
+   int def_x_neg=0, def_y_neg=0, paper_size_set;
    struct stat stat_buf;
    XWMHints wmhints;
    XSizeHints sizehints;
@@ -1162,6 +1233,12 @@ void Setup()
          noStatusWindow = TRUE;
       }
    }
+   noChoiceWindow = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoChoiceWindow")) != NULL) {
+      if (UtilStrICmp("true", c_ptr) == 0) {
+         noChoiceWindow = TRUE;
+      }
+   }
    noModeWindow = FALSE;
    noChatWindow = TRUE;
 
@@ -1447,42 +1524,8 @@ void Setup()
    if (*bootDir == '\0') {
       GetWorkingDirectory(bootDir, sizeof(bootDir));
    }
-   /* do not translate -- program constants */
-   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "TmpDir")) != NULL) {
-      UtilTrimBlanks(c_ptr);
-      if (*c_ptr == '\0') {
-         fprintf(stderr, TgLoadString(STID_INVALID_DIR_IN_X_DEFAULT),
-               c_ptr, TOOL_NAME, "TmpDir");
-         fprintf(stderr, "\n");
-      } else {
-         UtilStrCpyN(tmpDir, sizeof(tmpDir)-1, c_ptr);
-         strcat(tmpDir, DIR_SEP_STR);
-         if (UtilIsDirectory(tmpDir)) {
-            tmp_dir_in_x_def = TRUE;
-         } else {
-            fprintf(stderr, TgLoadString(STID_TMP_DIR_NOT_EXIST_USE_XDEF),
-                  tmpDir, TOOL_NAME, "TmpDir");
-            fprintf(stderr, "\n");
-         }
-      }
-   }
-   if (!tmp_dir_in_x_def) {
-#ifndef _TMP_DIR_IN_HOME_DIR
-      UtilStrCpyN(tmpDir, sizeof(tmpDir), TMP_DIR);
-      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "TmpDirInHomeDir")) !=
-            NULL && UtilStrICmp(c_ptr, "true") == 0) {
-         UtilStrCpyN(tmpDir, sizeof(tmpDir)-1, tgifDir);
-         strcat(tmpDir, DIR_SEP_STR);
-      }
-#else /* _TMP_DIR_IN_HOME_DIR */
-      UtilStrCpyN(tmpDir, sizeof(tmpDir)-1, tgifDir);
-      strcat(tmpDir, DIR_SEP_STR);
-      if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "TmpDirInHomeDir")) !=
-            NULL && UtilStrICmp(c_ptr, "false") == 0) {
-         UtilStrCpyN(tmpDir, sizeof(tmpDir), TMP_DIR);
-      }
-#endif /* ~_TMP_DIR_IN_HOME_DIR */
-   }
+   InitTmpDir();
+
    enableMouseWheel = TRUE;
    if (threeDLook && (c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
          "EnableMouseWheel")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
@@ -1572,6 +1615,7 @@ void Setup()
    if (canvasWindowOnly) {
       noMenubar = TRUE;
       noStatusWindow = TRUE;
+      noChoiceWindow = TRUE;
       noModeWindow = TRUE;
       noChatWindow = TRUE;
 
@@ -1608,8 +1652,9 @@ void Setup()
       sizehints.min_height =
             (noMenubar ? 0 : menubarWindowH+(brdrW<<1))+
             (noStatusWindow ? 0 : statusWindowH+(brdrW<<1))+
+            (noChoiceWindow ? 0 : msgWindowH+(brdrW<<1))+
             (noChatWindow ? 0 : chatWindowH+(brdrW<<1))+
-            titleWindowH+choiceWindowH+ONE_INCH+rulerW+scrollBarW+10*brdrW;
+            titleWindowH+ONE_INCH+rulerW+scrollBarW+8*brdrW;
       if (mainWinW > sizehints.min_width) {
          sizehints.width = mainWinW;
       } else {
@@ -1641,10 +1686,10 @@ void Setup()
       XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
    }
    InitWhiteBoard();
-   InitScroll();
-   InitPattern();
-   InitRuler();
    InitMenu();
+   InitPattern();
+
+   InitRuler();
    InitNames();
    InitStk();
    InitXBm();
@@ -1679,8 +1724,9 @@ void Setup()
          msgWindowW, msgWindowH, brdrW, myBorderPixel, myBgPixel)) == 0) {
       FailToCreateWindowMessage("Setup()", NULL, TRUE);
    }
-   cur_y += msgWindowH + (brdrW<<1);
-
+   if (!noChoiceWindow) {
+      cur_y += msgWindowH + (brdrW<<1);
+   }
    InitChoice();
 
    if ((modeWindow=XCreateSimpleWindow(mainDisplay, mainWindow, 0,
@@ -1760,6 +1806,7 @@ void Setup()
          brdrW, myBorderPixel, bg_pixel)) == 0) {
       FailToCreateWindowMessage("Setup()", NULL, TRUE);
    }
+   InitScroll();
    CalcStatusSubWinGeom();
    for (i=0; i < MAX_STATUS_BTNS; i++) {
       if ((statusSubWindow[i]=XCreateSimpleWindow(mainDisplay,
@@ -1817,7 +1864,9 @@ void Setup()
 
    InitExportFilters();
    InitProperties();
+   InitCutPaste();
    InitEdit();
+   InitMark();
    InitVersion();
    InitMiniLines();
 
@@ -1838,6 +1887,7 @@ void Setup()
    InitMeasureTooltip();
    InitPoly();
    InitSpecial();
+   InitTalkToSelfFiledes();
 
 #ifdef MAPBEFORESELECT
    XMapWindow(mainDisplay, mainWindow);
@@ -1872,12 +1922,12 @@ void Setup()
             EnterWindowMask | LeaveWindowMask | ButtonPressMask |
             PointerMotionMask);
 
-      XMapWindow(mainDisplay, msgWindow);
+      if (!noChoiceWindow) XMapWindow(mainDisplay, msgWindow);
       XSelectInput(mainDisplay, msgWindow, ButtonPressMask |
             ButtonReleaseMask | PointerMotionMask | ExposureMask |
             EnterWindowMask);
 
-      XMapWindow(mainDisplay, choiceWindow);
+      if (!noChoiceWindow) XMapWindow(mainDisplay, choiceWindow);
       XSelectInput(mainDisplay, choiceWindow,
             ButtonReleaseMask | ButtonPressMask | ExposureMask |
             PointerMotionMask | EnterWindowMask | LeaveWindowMask);
@@ -1970,12 +2020,12 @@ void Setup()
       XSelectInput(mainDisplay, msgWindow, ButtonPressMask |
             ButtonReleaseMask | PointerMotionMask | ExposureMask |
             EnterWindowMask);
-      XMapWindow(mainDisplay, msgWindow);
+      if (!noChoiceWindow) XMapWindow(mainDisplay, msgWindow);
 
       XSelectInput(mainDisplay, choiceWindow,
             ButtonReleaseMask | ButtonPressMask | ExposureMask |
             PointerMotionMask | EnterWindowMask | LeaveWindowMask);
-      XMapWindow(mainDisplay, choiceWindow);
+      if (!noChoiceWindow) XMapWindow(mainDisplay, choiceWindow);
 
       XSelectInput(mainDisplay, hRuleWindow,
             ButtonPressMask | ButtonReleaseMask | PointerMotionMask |
