@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.70 2000/08/04 13:39:13 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/shortcut.c,v 4.82 2001/10/14 00:20:02 william Exp $
  */
 
 #define _INCLUDE_FROM_SHORTCUT_C_
@@ -42,6 +42,11 @@
 #include "strtbl.e"
 #include "tangram2.e"
 
+/*
+ * Note:  the num_args field in shortCutXlateTbl is used by the shortcut
+ *        mechanism.  This includes the call_one_arg_shortcut() internal
+ *        command.
+ */
 /* do not translate -- program constants */
 static struct ShortCutRec shortCutXlateTbl[] = {
 /* Control Keys */
@@ -56,7 +61,7 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { 'k', CNTRL_ONLY, "Pop()", 0, CMDID_POP },
    { 'l', CNTRL_ONLY, "AlignObjs()", 0, CMDID_ALIGNOBJS },
    { 'n', CNTRL_ONLY, "New()", 0, CMDID_NEW },
-   { 'o', CNTRL_ONLY, "Open()", 0, CMDID_OPEN },
+   { 'o', CNTRL_ONLY, "Open()", 1, CMDID_OPEN },
    { 'p', CNTRL_ONLY, "Print()", 0, CMDID_PRINT },
    { 'q', CNTRL_ONLY, "Quit()", 0, CMDID_QUIT },
    { 'r', CNTRL_ONLY, "Redraw()", 0, CMDID_REDRAW },
@@ -125,7 +130,7 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '$', META_ONLY,  "SelectVertexMode()", 0, CMDID_SELECTVERTEXMODE },
    { '&', META_ONLY,  "AlignToPage()", 0, CMDID_ALIGNTOPAGE },
    { '*', META_ONLY,  "ChangeDomain()", 0, CMDID_CHANGEDOMAIN },
-   { '(', META_ONLY,  "ImportEPSFile()", 0, CMDID_IMPORTEPSFILE },
+   { '(', META_ONLY,  "ImportEPSFile()", 1, CMDID_IMPORTEPSFILE },
    { ')', META_ONLY,  "PreciseScale()", 0, CMDID_PRECISESCALE },
    { '<', META_ONLY,  "Lock()", 0, CMDID_LOCK },
    { '>', META_ONLY,  "UnLock()", 0, CMDID_UNLOCK },
@@ -190,7 +195,7 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '\0', 0, "NextPage()", 0, CMDID_NEXTPAGE },
    { '\0', 0, "PrevPage()", 0, CMDID_PREVPAGE },
    { '\0', 0, "NamePages()", 0, CMDID_NAMEPAGES },
-   { '\0', 0, "GotoPage()", 0, CMDID_GOTOPAGE },
+   { '\0', 0, "GotoPage()", 1, CMDID_GOTOPAGE },
    { '\0', 0, "AddPageBefore()", 0, CMDID_ADDPAGEBEFORE },
    { '\0', 0, "AddPageAfter()", 0, CMDID_ADDPAGEAFTER },
    { '\0', 0, "DeleteCurPage()", 0, CMDID_DELETECURPAGE },
@@ -375,6 +380,7 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '\0', 0, "ImportPNGFile()", 0, CMDID_IMPORTPNGFILE },
    { '\0', 0, "EditIndexedAttrInEditor()", 0, CMDID_EDITATTRINEDITOR },
    { '\0', 0, "GetProperty()", 0, CMDID_GETPROPERTY },
+   { '\0', 0, "PeekDimension()", 0, CMDID_PEEKDIMENSION },
    { '\0', 0, "SetHtmlExportTemplate()", 0, CMDID_SETHTMLEXPORTTEMPLATE },
    { '\0', 0, "PrintPages()", 0, CMDID_PRINTPAGES },
    { '\0', 0, "GoHyperSpaceInSlideShow()",0, CMDID_GOHYPERSPACEINSLIDESHOW },
@@ -392,6 +398,34 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '\0', 0, "RenameSignalNameForAPort()", 0, CMDID_RENAMESIGNAMEFORAPORT },
    { '\0', 0, "ClearSignalNameForAPort()", 0, CMDID_CLEARSIGNAMEFORAPORT },
    { '\0', 0, "ToggleShowWireSignalName()", 0, CMDID_TOGGLESHOWWIRESIGNALNAME },
+   { '\0', 0, "ToggleShowChoicebar()", 0, CMDID_TOGGLESHOWCHOICEBAR },
+   { '\0', 0, "MergePortsWithAnObject()", 0, CMDID_MERGEPORTSWITHANOBJECT },
+   { '\0', 0, "RenumberObjectIds()", 0, CMDID_RENUMBER_OBJECT_IDS },
+   { '\0', 0, "RepeatConnectTwoPortsByAWire()", 0,
+               CMDID_REPEATCONNECT2PORTSBYWIRE },
+   { '\0', 0, "ConnectPortsToBroadcastWire()", 0,
+               CMDID_CONNECTPORTSTOBROADCAST },
+   { '\0', 0, "ImportMultipageTextFile()", 0, CMDID_IMPORTMULTIPAGETEXTFILE },
+   { '\0', 0, "SetMarginsForImportMultipageTextFile()", 0,
+               CMDID_SETMARGINSONIMPORTTEXT },
+   { '\0', 0, "ToggleWordWrapDuringImportMultipageTextFile()", 0,
+               CMDID_TOGGLEWORDWRAPONIMPORTTEXT },
+   { '\0', 0, "HandleDataInMBuff()", 0, CMDID_DATA_IN_MBUFF },
+   { '\0', 0, "BenchMark()", 0, CMDID_BENCHMARK },
+
+   { '\0', 0, "ConvertToBezier()", 0, CMDID_CONVERTTOBEZIER },
+   { '\0', 0, "SetBezierConvertNumSegs()", 1, CMDID_SETBEZIERCONVERTNUMSEGS },
+
+   { '\0', 0, "AddSquareTickMarks()", 0, CMDID_ADDSQUARETICKMARKS },
+   { '\0', 0, "AddTriangleTickMarks()", 0, CMDID_ADDTRIANGLETICKMARKS },
+   { '\0', 0, "AddCircleTickMarks()", 0, CMDID_ADDCIRCLETICKMARKS },
+   { '\0', 0, "AddXTickMarks()", 0, CMDID_ADDXTICKMARKS },
+   { '\0', 0, "AddDiamondTickMarks()", 0, CMDID_ADDDIAMONDTICKMARKS },
+   { '\0', 0, "AddBowtieTickMarks()", 0, CMDID_ADDBOWTIETICKMARKS },
+   { '\0', 0, "AddInvTriangleTickMarks()", 0, CMDID_ADDINVTRIANGLETICKMARKS },
+   { '\0', 0, "AddPlusTickMarks()", 0, CMDID_ADDPLUSTICKMARKS },
+   { '\0', 0, "AddHourGlassTickMarks()", 0, CMDID_ADDHOURGLASSTICKMARKS },
+   { '\0', 0, "SetTickMarkSize()", 1, CMDID_SETTICKMARKSIZE },
    { '\0', 0, "", 0, 0 }
 };
 
