@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.43 2000/08/17 23:06:32 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.56 2001/08/06 21:39:14 william Exp $
  */
 
 #define _INCLUDE_FROM_SPECIAL_C_
@@ -76,9 +76,11 @@
 #include "stk.e"
 #include "stretch.e"
 #include "strtbl.e"
+#include "tangram2.e"
 #include "text.e"
 #include "util.e"
 #include "version.e"
+#include "wb.e"
 
 int placingTopObj=FALSE;
 int connectingPortsByWire=FALSE;
@@ -343,9 +345,14 @@ unsigned int PlaceTopObj(ObjPtr)
          DrawObj(drawWindow, ObjPtr);
          HideInterrupt();
       } else if (input.type == MotionNotify) {
+         int anywhere=(input.xmotion.state & (ShiftMask | ControlMask));
+         int saved_snap=snapOn;
+
          x = input.xmotion.x;
          y = input.xmotion.y;
+         if (anywhere) snapOn = FALSE;
          GridXY(x, y, &grid_x, &grid_y);
+         if (anywhere) snapOn = saved_snap;
          SelBox(drawWindow, revDefaultGC, obj_ltx+grid_dx+dx,
                obj_lty+grid_dy+dy, obj_rbx+grid_dx+dx, obj_rby+grid_dy+dy);
          dx = grid_x - orig_x;
@@ -609,6 +616,13 @@ int ObjIsAPort(obj_ptr)
                   return FALSE;
                }
             } else {
+               if (owner_obj->type == OBJ_GROUP) {
+                  attr_ptr = FindAttrWithName(owner_obj, "type=", NULL);
+                  if (attr_ptr != NULL && strcmp(attr_ptr->attr_value.s,
+                        "tgBroadcastWire") == 0) {
+                     return TRUE;
+                  }
+               }
                return FALSE;
             }
          }
@@ -716,9 +730,14 @@ void AddWireAttributes(poly_obj_ptr, signal_name, p_obbox, auto_center)
 }
 
 typedef struct tagChangeConnectionViewInfo {
+   /* only change view if do_change_view is TRUE */
+   int do_change_view;
    int change_to_connect_view;
    struct ObjRec *conn_view_obj, *disconn_view_obj;
    int conn_fill, conn_pen, disconn_fill, disconn_pen;
+   /* _TANGRAM_II stuff */
+   struct ObjRec *port_obj;
+   char *signal_name;
 } ChangeConnectionViewInfo;
 
 static
@@ -730,17 +749,23 @@ int ChangeConnectionView(pravci)
 
    if (pccvi == NULL) return FALSE;
 
-   if (pccvi->change_to_connect_view) {
-      ChangeObjFill(pccvi->disconn_view_obj, NONEPAT);
-      ChangeObjPen(pccvi->disconn_view_obj, NONEPAT);
-      ChangeObjFill(pccvi->conn_view_obj, pccvi->conn_fill);
-      ChangeObjPen(pccvi->conn_view_obj, pccvi->conn_pen);
-   } else {
-      ChangeObjFill(pccvi->conn_view_obj, NONEPAT);
-      ChangeObjPen(pccvi->conn_view_obj, NONEPAT);
-      ChangeObjFill(pccvi->disconn_view_obj, pccvi->disconn_fill);
-      ChangeObjPen(pccvi->disconn_view_obj, pccvi->disconn_pen);
+   if (pccvi->do_change_view) {
+      if (pccvi->change_to_connect_view) {
+         ChangeObjFill(pccvi->disconn_view_obj, NONEPAT);
+         ChangeObjPen(pccvi->disconn_view_obj, NONEPAT);
+         ChangeObjFill(pccvi->conn_view_obj, pccvi->conn_fill);
+         ChangeObjPen(pccvi->conn_view_obj, pccvi->conn_pen);
+      } else {
+         ChangeObjFill(pccvi->conn_view_obj, NONEPAT);
+         ChangeObjPen(pccvi->conn_view_obj, NONEPAT);
+         ChangeObjFill(pccvi->disconn_view_obj, pccvi->disconn_fill);
+         ChangeObjPen(pccvi->disconn_view_obj, pccvi->disconn_pen);
+      }
    }
+#ifdef _TANGRAM_II
+   Tangram2UpdateInitAttr(pccvi->port_obj, pccvi->signal_name);
+#endif /* _TANGRAM_II */
+
    return TRUE;
 }
 
@@ -797,9 +822,9 @@ int HasConnAndDisConnViewObjSubObjs(obj_ptr, pccvi)
 }
 
 static
-int SetPortSignalName(attr_ptr, attr_owner_obj, new_signal_name)
+int SetPortSignalName(port_obj, attr_ptr, attr_owner_obj, new_signal_name)
+   struct ObjRec *port_obj, *attr_owner_obj;
    struct AttrRec *attr_ptr;
-   struct ObjRec *attr_owner_obj;
    char *new_signal_name;
 {
    if (attr_ptr != NULL &&
@@ -810,6 +835,7 @@ int SetPortSignalName(attr_ptr, attr_owner_obj, new_signal_name)
       memset(&ccvi, 0, sizeof(ChangeConnectionViewInfo));
       if ((*old_signal_name == '\0' || *new_signal_name == '\0') &&
             HasConnAndDisConnViewObjSubObjs(attr_ptr->owner, &ccvi)) {
+         ccvi.do_change_view = TRUE;
          if (*old_signal_name == '\0') {
             /* the port will become connected */
             ccvi.change_to_connect_view = TRUE;
@@ -817,9 +843,12 @@ int SetPortSignalName(attr_ptr, attr_owner_obj, new_signal_name)
             /* the port will become disconnected */
             ccvi.change_to_connect_view = FALSE;
          }
-         SetReplaceAttrValueCallback(
-               (ReplAttrValueCallback*)ChangeConnectionView, &ccvi);
       }
+      ccvi.port_obj = port_obj;
+      ccvi.signal_name = new_signal_name;
+      SetReplaceAttrValueCallback(
+            (ReplAttrValueCallback*)ChangeConnectionView, &ccvi);
+
       ReplaceAttrFirstValue(attr_owner_obj, attr_ptr, new_signal_name);
       SetReplaceAttrValueCallback(NULL, NULL);
       RecursivelyAdjObjBBox(attr_ptr->owner, attr_ptr->owner,
@@ -830,13 +859,15 @@ int SetPortSignalName(attr_ptr, attr_owner_obj, new_signal_name)
 }
 
 static
-void DoNameWire()
+void DoNameWire(automatic)
+   int automatic;
 {
    struct AttrRec *first_attr_ptr=NULL, *last_attr_ptr=NULL;
    struct ObjRec *first_attr_owner_obj=NULL, *last_attr_owner_obj=NULL;
    struct BBRec bbox;
    char signal_name[MAXSTRING], *existing_signal_name=NULL;
-   int need_to_clear=FALSE, do_prompt=TRUE, need_to_redraw=FALSE;
+   int need_to_clear=FALSE, need_to_redraw=FALSE;
+   int do_prompt=TRUE, must_prompt=FALSE;
 
    memcpy(&bbox, &topObj->bbox, sizeof(struct BBRec));
    first_attr_ptr = FindAttrWithName(gstWiringInfo.first_port_obj,
@@ -857,7 +888,7 @@ void DoNameWire()
             0) {
          if (*first_attr_ptr->attr_value.s == '\0' &&
                *last_attr_ptr->attr_value.s == '\0') {
-            do_prompt = TRUE;
+            must_prompt = TRUE;
          } else if (*first_attr_ptr->attr_value.s == '\0') {
             existing_signal_name = last_attr_ptr->attr_value.s;
          } else if (*last_attr_ptr->attr_value.s == '\0') {
@@ -869,7 +900,7 @@ void DoNameWire()
             switch (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB)) {
             case MB_ID_YES:
                need_to_clear = TRUE;
-               do_prompt = TRUE;
+               must_prompt = TRUE;
                break;
             case MB_ID_NO:
             case MB_ID_CANCEL:
@@ -890,11 +921,13 @@ void DoNameWire()
    }
    StartCompositeCmd();
    if (need_to_clear) {
-      if (SetPortSignalName(first_attr_ptr, first_attr_owner_obj, "")) {
+      if (SetPortSignalName(gstWiringInfo.first_port_obj, first_attr_ptr,
+            first_attr_owner_obj, "")) {
          UnionRect(&bbox, &first_attr_owner_obj->bbox, &bbox);
          need_to_redraw = TRUE;
       }
-      if (SetPortSignalName(last_attr_ptr, last_attr_owner_obj, "")) {
+      if (SetPortSignalName(gstWiringInfo.last_port_obj, last_attr_ptr,
+            last_attr_owner_obj, "")) {
          UnionRect(&bbox, &last_attr_owner_obj->bbox, &bbox);
          need_to_redraw = TRUE;
       }
@@ -903,13 +936,17 @@ void DoNameWire()
       struct ObjRec *signal_name_obj=NULL;
       struct BBRec obbox;
       XEvent ev;
+      int show_wire_signal_name=(!automatic && showWireSignalName);
 
       *signal_name = '\0';
       if (existing_signal_name != NULL) {
          UtilStrCpyN(signal_name, sizeof(signal_name), existing_signal_name);
       }
-      Dialog(TgLoadString(STID_PLS_ENT_SIG_NAME), NULL, signal_name);
       UtilTrimBlanks(signal_name);
+      if (must_prompt || !automatic) {
+         Dialog(TgLoadString(STID_PLS_ENT_SIG_NAME), NULL, signal_name);
+         UtilTrimBlanks(signal_name);
+      }
       if (*signal_name == '\0') {
          if (need_to_clear) {
             sprintf(gszMsgBox, TgLoadString(STID_SIGNAL_NAMES_CLEARED),
@@ -929,14 +966,18 @@ void DoNameWire()
          }
          return;
       }
-      if (showWireSignalName) {
+      if (show_wire_signal_name) {
+         unsigned int button_pressed=Button1;
+
          SaveStatusStrings();
          sprintf(gszMsgBox, TgLoadString(STID_PLACING_NAMED_SIGNAL),
                signal_name);
          SetStringStatus(gszMsgBox);
-         PasteString(signal_name, FALSE, FALSE);
+         button_pressed = PasteString(signal_name, FALSE, FALSE);
          RestoreStatusStrings();
 
+         if (button_pressed != Button1) show_wire_signal_name = FALSE;
+
          memcpy(&obbox, &topObj->obbox, sizeof(struct BBRec));
          signal_name_obj = topObj;
          UnlinkObj(signal_name_obj);
@@ -949,14 +990,15 @@ void DoNameWire()
       } else {
          memcpy(&obbox, &topObj->obbox, sizeof(struct BBRec));
       }
-      AddWireAttributes(topObj, signal_name, &obbox, !showWireSignalName);
+      AddWireAttributes(topObj, signal_name, &obbox, !show_wire_signal_name);
       UnionRect(&bbox, &topObj->bbox, &bbox);
-      if (SetPortSignalName(first_attr_ptr, first_attr_owner_obj,
-            signal_name)) {
+      if (SetPortSignalName(gstWiringInfo.first_port_obj, first_attr_ptr,
+            first_attr_owner_obj, signal_name)) {
          UnionRect(&bbox, &first_attr_owner_obj->bbox, &bbox);
          need_to_redraw = TRUE;
       }
-      if (SetPortSignalName(last_attr_ptr, last_attr_owner_obj, signal_name)) {
+      if (SetPortSignalName(gstWiringInfo.last_port_obj, last_attr_ptr,
+            last_attr_owner_obj, signal_name)) {
          UnionRect(&bbox, &last_attr_owner_obj->bbox, &bbox);
          need_to_redraw = TRUE;
       }
@@ -971,14 +1013,378 @@ void DoNameWire()
             bbox.lty-QUARTER_INCH-GRID_ABS_SIZE(1),
             bbox.rbx+QUARTER_INCH+GRID_ABS_SIZE(1),
             bbox.rby+QUARTER_INCH+GRID_ABS_SIZE(1));
+      if (topSel!=NULL) HighLightForward();
    }
 }
 
 void ConnectTwoPortsByAWire()
 {
+   StartCompositeCmd();
    if (DoConnectTwoPorts()) {
-      DoNameWire();
+      DoNameWire(FALSE);
    }
+   EndCompositeCmd();
+}
+
+void RepeatConnectTwoPortsByAWire()
+{
+   StartCompositeCmd();
+   while (DoConnectTwoPorts()) {
+      DoNameWire(TRUE);
+
+      if (topSel!=NULL) { HighLightReverse(); RemoveAllSel(); }
+   }
+   EndCompositeCmd();
+}
+
+static
+void AddObjToAnchor(anchor, obj_ptr)
+   struct ObjRec *anchor, *obj_ptr;
+{
+   obj_ptr->prev = NULL;
+   obj_ptr->next = anchor->detail.r->first;
+
+   anchor->detail.r->first->prev = obj_ptr;
+   anchor->detail.r->first = obj_ptr;
+   AdjObjOBBox(anchor);
+}
+
+static
+void AddBroadcastWireAttributes(poly_obj_ptr, signal_name, p_obbox, auto_center)
+   struct ObjRec *poly_obj_ptr;
+   char *signal_name;
+   struct BBRec *p_obbox;
+   int auto_center;
+{
+   struct ObjRec *text_obj_ptr=NULL;
+
+   PrepareToReplaceAnObj(poly_obj_ptr);
+
+   AddAttrByNameAndValue(poly_obj_ptr, "broadcast_signal_name=", signal_name);
+   poly_obj_ptr->fattr->shown = TRUE;
+   poly_obj_ptr->fattr->nameshown = FALSE;
+   UpdAttr(poly_obj_ptr->fattr);
+   text_obj_ptr = poly_obj_ptr->fattr->obj;
+   if (auto_center) {
+      CenterObjInOBBox(text_obj_ptr, *p_obbox, NULL);
+      memcpy(p_obbox, &text_obj_ptr->obbox, sizeof(struct BBRec));
+      poly_obj_ptr->fattr->shown = FALSE;
+   } else {
+      MoveObj(text_obj_ptr, p_obbox->ltx-text_obj_ptr->obbox.ltx,
+            p_obbox->lty-text_obj_ptr->obbox.lty);
+   }
+   AdjObjBBox(poly_obj_ptr);
+
+   RecordReplaceAnObj(poly_obj_ptr);
+}
+
+static
+void DoNameBroadcastWire(signal_name, signal_name_diff)
+   char *signal_name;
+   int signal_name_diff;
+{
+   struct ObjRec *obj_ptr=NULL, *signal_name_obj=NULL;
+   struct AttrRec *attr_ptr=NULL;
+   struct BBRec bbox, obbox;
+   int show_wire_signal_name=showWireSignalName;
+   int already_has_broadcast_signal_name=FALSE;
+   XEvent ev;
+
+   memset(&obbox, 0, sizeof(obbox));
+   memcpy(&bbox, &topObj->bbox, sizeof(struct BBRec));
+
+   StartCompositeCmd();
+
+   attr_ptr = FindAttrWithName(topObj, "broadcast_signal_name=", NULL);
+   if (attr_ptr != NULL) {
+      already_has_broadcast_signal_name = TRUE;
+      if (strcmp(attr_ptr->attr_value.s, signal_name) != 0) {
+         ReplaceAttrFirstValue(topObj, attr_ptr, signal_name);
+         UnionRect(&bbox, &topObj->bbox, &bbox);
+      }
+   }
+   for (obj_ptr=topObj->detail.r->last; obj_ptr != NULL;
+         obj_ptr=obj_ptr->prev) {
+      if (ObjIsAPort(obj_ptr)) {
+         struct ObjRec *attr_owner_obj=NULL;
+
+         attr_ptr = FindAttrWithName(obj_ptr, "signal_name=", &attr_owner_obj);
+         while (attr_owner_obj != NULL && attr_owner_obj->tmp_parent != NULL) {
+            attr_owner_obj = attr_owner_obj->tmp_parent;
+         }
+         if (SetPortSignalName(obj_ptr, attr_ptr, attr_owner_obj, "")) {
+            UnionRect(&bbox, &attr_owner_obj->bbox, &bbox);
+         }
+      }
+   }
+   if (show_wire_signal_name && !already_has_broadcast_signal_name) {
+      unsigned int button_pressed=Button1;
+
+      SaveStatusStrings();
+      sprintf(gszMsgBox, TgLoadString(STID_PLACING_NAMED_SIGNAL),
+            signal_name);
+      SetStringStatus(gszMsgBox);
+      button_pressed = PasteString(signal_name, FALSE, FALSE);
+      RestoreStatusStrings();
+
+      if (button_pressed != Button1) show_wire_signal_name = FALSE;
+
+      memcpy(&obbox, &topObj->obbox, sizeof(struct BBRec));
+      signal_name_obj = topObj;
+      UnlinkObj(signal_name_obj);
+      FreeObj(signal_name_obj);
+
+      XSync(mainDisplay, False);
+      if (XCheckMaskEvent(mainDisplay, ExposureMask, &ev)) {
+         ExposeEventHandler(&ev, TRUE);
+      }
+   } else {
+      memcpy(&obbox, &topObj->obbox, sizeof(struct BBRec));
+   }
+   if (!already_has_broadcast_signal_name) {
+      AddBroadcastWireAttributes(topObj, signal_name, &obbox,
+            !show_wire_signal_name);
+      UnionRect(&bbox, &topObj->bbox, &bbox);
+   }
+   for (obj_ptr=topObj->detail.r->last; obj_ptr != NULL;
+         obj_ptr=obj_ptr->prev) {
+      if (ObjIsAPort(obj_ptr)) {
+         struct ObjRec *attr_owner_obj=NULL;
+         struct AttrRec *attr_ptr=NULL;
+
+         attr_ptr = FindAttrWithName(obj_ptr, "signal_name=", &attr_owner_obj);
+         while (attr_owner_obj != NULL && attr_owner_obj->tmp_parent != NULL) {
+            attr_owner_obj = attr_owner_obj->tmp_parent;
+         }
+         if (SetPortSignalName(obj_ptr, attr_ptr, attr_owner_obj,
+               signal_name)) {
+            UnionRect(&bbox, &attr_owner_obj->bbox, &bbox);
+         }
+      }
+   }
+   EndCompositeCmd();
+
+   SetFileModified(TRUE);
+   justDupped = FALSE;
+   RedrawAnArea(botObj,
+         bbox.ltx-QUARTER_INCH-GRID_ABS_SIZE(1),
+         bbox.lty-QUARTER_INCH-GRID_ABS_SIZE(1),
+         bbox.rbx+QUARTER_INCH+GRID_ABS_SIZE(1),
+         bbox.rby+QUARTER_INCH+GRID_ABS_SIZE(1));
+   if (topSel!=NULL) HighLightForward();
+}
+
+static
+void TrySetCommonSignalName(attr_ptr, pszSignalName, szSignalName,
+      pnSignalNameDiff)
+   struct AttrRec *attr_ptr;
+   char *pszSignalName;
+   int szSignalName, *pnSignalNameDiff;
+{
+   if (*pnSignalNameDiff) {
+      /* already see multiple signal names */
+   } else if (*attr_ptr->attr_value.s == '\0') {
+      /* nothing to do */
+   } else if (*pszSignalName == '\0') {
+      UtilStrCpyN(pszSignalName, szSignalName, attr_ptr->attr_value.s);
+   } else if (strcmp(pszSignalName, attr_ptr->attr_value.s) != 0) {
+      (*pnSignalNameDiff) = TRUE;
+      *pszSignalName = '\0';
+   }
+}
+
+static
+int CheckPortsAndBroadcastObj(ppAnchor, pnAnchorIsPoly, pnNumPorts,
+      pszSignalName, szSignalName, pnSignalNameDiff,
+      pnAlreadyHasBroadcastSignalName)
+   struct ObjRec **ppAnchor;
+   int *pnAnchorIsPoly, *pnNumPorts, szSignalName, *pnSignalNameDiff;
+   int *pnAlreadyHasBroadcastSignalName;
+   char *pszSignalName;
+{
+   int num_ports=0, anchor_is_poly=FALSE, ok=TRUE, signal_name_diff=FALSE;
+   int already_has_broadcast_signal_name=FALSE;
+   struct ObjRec *anchor=NULL;
+   struct SelRec *sel_ptr=NULL;
+
+   *pszSignalName = '\0';
+   for (sel_ptr=botSel; ok && sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
+      struct AttrRec *attr_ptr=NULL;
+
+      switch (sel_ptr->obj->type) {
+      case OBJ_GROUP:
+      case OBJ_SYM:
+      case OBJ_ICON:
+      case OBJ_PIN:
+         if ((attr_ptr=FindAttrWithName(sel_ptr->obj, "type=", NULL)) != NULL) {
+            if (strcmp(attr_ptr->attr_value.s, "port") == 0) {
+               if ((attr_ptr=FindAttrWithName(sel_ptr->obj, "signal_name=",
+                     NULL)) != NULL) {
+                  TrySetCommonSignalName(attr_ptr, pszSignalName, szSignalName,
+                        &signal_name_diff);
+                  num_ports++;
+               }
+            } else if (strcmp(attr_ptr->attr_value.s, "tgBroadcastWire") == 0) {
+               anchor = sel_ptr->obj;
+               anchor_is_poly = FALSE;
+               if ((attr_ptr=FindAttrWithName(sel_ptr->obj,
+                     "broadcast_signal_name=", NULL)) != NULL) {
+                  TrySetCommonSignalName(attr_ptr, pszSignalName, szSignalName,
+                        &signal_name_diff);
+                  already_has_broadcast_signal_name = TRUE;
+               }
+            } else {
+               return FALSE;
+            }
+         } else {
+            return FALSE;
+         }
+         break;
+      case OBJ_POLY:
+         if (anchor == NULL) {
+            anchor = sel_ptr->obj;
+            anchor_is_poly = TRUE;
+         } else {
+            return FALSE;
+         }
+         break;
+      default:
+         return FALSE;
+         break;
+      }
+   }
+   if (anchor == NULL || num_ports == 0) {
+      return FALSE;
+   }
+   *pnNumPorts = num_ports;
+   *ppAnchor = anchor;
+   *pnAnchorIsPoly = anchor_is_poly;
+   *pnSignalNameDiff = signal_name_diff;
+   *pnAlreadyHasBroadcastSignalName = already_has_broadcast_signal_name;
+
+   return TRUE;
+}
+
+void ConnectPortsToBroadcastWire()
+{
+   struct ObjRec *anchor=NULL;
+   struct SelRec *sel_ptr=NULL;
+   int num_ports=0, anchor_is_poly=FALSE, signal_name_diff=FALSE;
+   int sel_ltx=selLtX, sel_lty=selLtY, sel_rbx=selRbX, sel_rby=selRbY;
+   int already_has_broadcast_signal_name=FALSE;
+   char existing_signal_name[MAXSTRING], new_signal_name[MAXSTRING];
+
+   *existing_signal_name = '\0';
+   if (!CheckPortsAndBroadcastObj(&anchor, &anchor_is_poly, &num_ports,
+         existing_signal_name, sizeof(existing_signal_name),
+         &signal_name_diff, &already_has_broadcast_signal_name)) {
+      MsgBox(TgLoadString(STID_SEL_ONE_NONE_PORT_PLUS_BRDCST), TOOL_NAME,
+            INFO_MB);
+      return;
+   }
+   *new_signal_name = '\0';
+   if (*existing_signal_name == '\0') {
+      if (signal_name_diff) {
+         /* conflicting signal names */
+         sprintf(gszMsgBox, TgLoadString(STID_CONFLICT_SIG_NAME_ENT_NEW));
+      } else {
+         /* all ports have no signal names */
+         sprintf(gszMsgBox, TgLoadString(STID_PLS_ENT_SIG_NAME));
+      }
+   } else {
+      UtilStrCpyN(new_signal_name, sizeof(new_signal_name),
+            existing_signal_name);
+      sprintf(gszMsgBox, TgLoadString(STID_PLS_ENT_SIG_NAME));
+   }
+   if (!(*new_signal_name != '\0' && already_has_broadcast_signal_name)) {
+      UtilTrimBlanks(new_signal_name);
+      Dialog(gszMsgBox, TgLoadCachedString(CSTID_DLG_ACCEPT_CANCEL),
+            new_signal_name);
+      UtilTrimBlanks(new_signal_name);
+      if (*new_signal_name == '\0') {
+         return;
+      }
+   }
+   StartCompositeCmd();
+
+   HighLightReverse();
+   if (anchor_is_poly) {
+      struct SelRec *saved_top_sel=NULL, *saved_bot_sel=NULL;
+      struct ObjRec *text_obj_ptr=NULL;
+      struct BBRec obbox;
+      int saved_num_obj_selected=0;
+      char name[40];
+
+      PrepareToReplaceAnObj(anchor);
+      for (sel_ptr=botSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
+         if (sel_ptr->obj == anchor) {
+            UnlinkSel(sel_ptr, &topSel, &botSel);
+            numObjSelected--;
+            break;
+         }
+      }
+      saved_top_sel = topSel;
+      saved_bot_sel = botSel;
+      saved_num_obj_selected = numObjSelected;
+      topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+      if (topSel == NULL) FailAllocMessage();
+      topSel->obj = anchor;
+      topSel->next = topSel->prev = NULL;
+      numObjSelected = 1;
+      GroupSingleObj(FALSE);
+      topSel = saved_top_sel;
+      botSel = saved_bot_sel;
+      numObjSelected = saved_num_obj_selected;
+      AddObjIntoSel(topObj, NULL, topSel, &topSel, &botSel);
+      numObjSelected++;
+      anchor = topObj;
+
+      memcpy(&obbox, &anchor->obbox, sizeof(struct BBRec));
+      AddAttrByNameAndValue(anchor, "type=", "tgBroadcastWire");
+      text_obj_ptr = anchor->fattr->obj;
+      MoveObj(text_obj_ptr, obbox.ltx-text_obj_ptr->obbox.ltx,
+            obbox.rby-text_obj_ptr->obbox.lty);
+      AdjObjBBox(anchor);
+
+      sprintf(name, "tgAutoObj%d", anchor->id);
+      memcpy(&obbox, &anchor->obbox, sizeof(struct BBRec));
+      AddAttrByNameAndValue(anchor, "name=", name);
+      text_obj_ptr = anchor->fattr->obj;
+      MoveObj(text_obj_ptr, obbox.ltx-text_obj_ptr->obbox.ltx,
+            obbox.rby-text_obj_ptr->obbox.lty);
+      AdjObjBBox(anchor);
+
+      RecordReplaceAnObj(anchor);
+   }
+   PrepareToRecord(CMD_REPLACE, topSel, botSel, numObjSelected);
+
+   for (sel_ptr=botSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
+      if (sel_ptr->obj != anchor) {
+         UnlinkObj(sel_ptr->obj);
+         AddObjToAnchor(anchor, sel_ptr->obj);
+      }
+   }
+   RemoveAllSel();
+   UnlinkObj(anchor);
+   AddObj(NULL, topObj, anchor);
+   topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (topSel == NULL) FailAllocMessage();
+   topSel->obj = anchor;
+   topSel->prev = NULL;
+   botSel->next = NULL;
+   AdjObjBBox(anchor);
+   UpdSelBBox();
+   RecordCmd(CMD_MANY_TO_ONE, NULL, topSel, botSel, 1);
+   RedrawAreas(botObj, sel_ltx-GRID_ABS_SIZE(1), sel_lty-GRID_ABS_SIZE(1),
+         sel_rbx+GRID_ABS_SIZE(1), sel_rby+GRID_ABS_SIZE(1),
+         selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
+         selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
+   HighLightForward();
+   SetFileModified(TRUE);
+   justDupped = FALSE;
+
+   DoNameBroadcastWire(new_signal_name, signal_name_diff);
+   EndCompositeCmd();
 }
 
 #define DO_RENAME_SIGNAL_NAME 99
@@ -990,6 +1396,7 @@ void DoNameOrClearAPort(which)
 {
    struct AttrRec *first_attr_ptr=NULL;
    struct ObjRec *first_attr_owner_obj=NULL;
+   int ltx=0, lty=0, rbx=0, rby=0;
    char signal_name[MAXSTRING];
    ChangeConnectionViewInfo ccvi;
 
@@ -1023,7 +1430,13 @@ void DoNameOrClearAPort(which)
    if (strcmp(signal_name, first_attr_ptr->attr_value.s) == 0) {
       return;
    }
+   ltx = first_attr_owner_obj->bbox.ltx;
+   lty = first_attr_owner_obj->bbox.lty;
+   rbx = first_attr_owner_obj->bbox.rbx;
+   rby = first_attr_owner_obj->bbox.rby;
+
    if (HasConnAndDisConnViewObjSubObjs(first_attr_ptr->owner, &ccvi)) {
+      ccvi.do_change_view = TRUE;
       if (*signal_name != '\0') {
          /* the port will become connected */
          ccvi.change_to_connect_view = TRUE;
@@ -1031,17 +1444,23 @@ void DoNameOrClearAPort(which)
          /* the port will become disconnected */
          ccvi.change_to_connect_view = FALSE;
       }
-      SetReplaceAttrValueCallback(
-            (ReplAttrValueCallback*)ChangeConnectionView, &ccvi);
    }
+   ccvi.port_obj = gstWiringInfo.first_port_obj;
+   ccvi.signal_name = signal_name;
+   SetReplaceAttrValueCallback(
+         (ReplAttrValueCallback*)ChangeConnectionView, &ccvi);
+
    ReplaceAttrFirstValue(first_attr_owner_obj, first_attr_ptr,
          signal_name);
+   SetReplaceAttrValueCallback(NULL, NULL);
    RecursivelyAdjObjBBox(first_attr_ptr->owner, first_attr_ptr->owner,
          first_attr_owner_obj);
 
    SetFileModified(TRUE);
    justDupped = FALSE;
-   RedrawAnArea(botObj,
+   RedrawAreas(botObj,
+         ltx-QUARTER_INCH-GRID_ABS_SIZE(1), lty-QUARTER_INCH-GRID_ABS_SIZE(1),
+         rbx+QUARTER_INCH+GRID_ABS_SIZE(1), rby+QUARTER_INCH+GRID_ABS_SIZE(1),
          first_attr_owner_obj->bbox.ltx-QUARTER_INCH-GRID_ABS_SIZE(1),
          first_attr_owner_obj->bbox.lty-QUARTER_INCH-GRID_ABS_SIZE(1),
          first_attr_owner_obj->bbox.rbx+QUARTER_INCH+GRID_ABS_SIZE(1),
@@ -1137,6 +1556,145 @@ void ClearSignalNameForAPort()
    RenameOrClearSignalNameForAPort(DO_CLEAR_SIGNAL_NAME);
 }
 
+static
+int CheckPortsAndObj(ppAnchor, pnNumPorts)
+   struct ObjRec **ppAnchor;
+   int *pnNumPorts;
+{
+   int num_ports=0, ok=TRUE;
+   struct ObjRec *anchor=NULL;
+   struct SelRec *sel_ptr=NULL;
+
+   for (sel_ptr=botSel; ok && sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
+      struct AttrRec *attr_ptr=NULL;
+
+      switch (sel_ptr->obj->type) {
+      case OBJ_GROUP:
+      case OBJ_SYM:
+      case OBJ_ICON:
+      case OBJ_PIN:
+         if ((attr_ptr=FindAttrWithName(sel_ptr->obj, "type=", NULL)) != NULL) {
+            if (strcmp(attr_ptr->attr_value.s, "port") == 0 &&
+                  FindAttrWithName(sel_ptr->obj, "signal_name=",
+                  NULL) != NULL) {
+               num_ports++;
+            } else if (anchor == NULL) {
+               anchor = sel_ptr->obj;
+            } else {
+               return FALSE;
+            }
+         } else if (anchor == NULL) {
+            anchor = sel_ptr->obj;
+         } else {
+            return FALSE;
+         }
+         break;
+      default:
+         return FALSE;
+         break;
+      }
+   }
+   if (anchor == NULL || num_ports == 0) {
+      return FALSE;
+   }
+   *pnNumPorts = num_ports;
+   *ppAnchor = anchor;
+
+   return TRUE;
+}
+
+void MergePortsWithAnObject()
+{
+   int num_ports=0;
+   int sel_ltx=selLtX, sel_lty=selLtY, sel_rbx=selRbX, sel_rby=selRbY;
+   struct ObjRec *anchor=NULL;
+   struct SelRec *sel_ptr=NULL;
+
+   if (!CheckPortsAndObj(&anchor, &num_ports)) {
+      MsgBox(TgLoadString(STID_SEL_ONE_NONE_PORT_PLUS_COMP), TOOL_NAME,
+            INFO_MB);
+      return;
+   }
+   HighLightReverse();
+   PrepareToRecord(CMD_REPLACE, topSel, botSel, numObjSelected);
+
+   for (sel_ptr=botSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
+      if (sel_ptr->obj != anchor) {
+         UnlinkObj(sel_ptr->obj);
+         AddObjToAnchor(anchor, sel_ptr->obj);
+      }
+   }
+   RemoveAllSel();
+   UnlinkObj(anchor);
+   AddObj(NULL, topObj, anchor);
+   topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (topSel == NULL) FailAllocMessage();
+   topSel->obj = anchor;
+   topSel->prev = NULL;
+   botSel->next = NULL;
+   AdjObjBBox(anchor);
+   UpdSelBBox();
+   RecordCmd(CMD_MANY_TO_ONE, NULL, topSel, botSel, 1);
+   RedrawAreas(botObj, sel_ltx-GRID_ABS_SIZE(1), sel_lty-GRID_ABS_SIZE(1),
+         sel_rbx+GRID_ABS_SIZE(1), sel_rby+GRID_ABS_SIZE(1),
+         selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
+         selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
+   HighLightForward();
+   SetFileModified(TRUE);
+   justDupped = FALSE;
+}
+
+static
+void RenumberObjId(obj_ptr)
+   struct ObjRec *obj_ptr;
+{
+   struct AttrRec *attr_ptr=NULL;
+   struct ObjRec *sub_obj=NULL;
+
+   obj_ptr->id = objId++;
+
+   switch (obj_ptr->type) {
+   case OBJ_GROUP:
+   case OBJ_ICON:
+   case OBJ_SYM:
+   case OBJ_PIN:
+      for (sub_obj=obj_ptr->detail.r->last; sub_obj != NULL;
+            sub_obj=sub_obj->prev) {
+         RenumberObjId(sub_obj);
+      }
+      break;
+   }
+   for (attr_ptr=obj_ptr->lattr; attr_ptr != NULL; attr_ptr=attr_ptr->prev) {
+      RenumberObjId(attr_ptr->obj);
+   }
+}
+
+void RenumberObjectIds()
+{
+   struct PageRec *page_ptr=NULL;
+
+   if (gstWBInfo.do_whiteboard) {
+      MsgBox(TgLoadString(STID_RENUMBER_OBJ_IDS_IN_WB), TOOL_NAME, INFO_MB);
+      return;
+   }
+   if (!OkToFlushUndoBuffer(TgLoadString(STID_RENUMBER_OBJ_IDS_CAUSE_FLUSH))) {
+      return;
+   }
+   MakeQuiescent();
+   CleanUpCmds();
+
+   objId = 0;
+   for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
+      struct ObjRec *obj_ptr=NULL;
+
+      for (obj_ptr=page_ptr->bot; obj_ptr != NULL; obj_ptr=obj_ptr->prev) {
+         RenumberObjId(obj_ptr);
+      }
+   }
+   MsgBox(TgLoadString(STID_RENUMBER_OBJ_IDS_DONE), TOOL_NAME, INFO_MB);
+   SetFileModified(TRUE);
+}
+
 void MakeSymbolic()
 {
    int ltx=selLtX, lty=selLtY, rbx=selRbX, rby=selRbY;
