@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/stream.c,v 4.12 2000/01/15 22:24:14 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/stream.c,v 4.19 2001/08/11 21:56:20 william Exp $
  */
 
 #define _INCLUDE_FROM_STREAM_C_
@@ -248,8 +248,9 @@ int ValidMultipartReplace(buf, pn_multipart)
 }
 
 static char *gppszValidImageContentType[] = {
-    "image/jpeg", "JPEG-dither", /* only knows about JPEG movie */
-    "image/jpg", "JPEG-dither", /* only knows about JPEG movie */
+    "image/jpeg", "JPEG-dither", /* JPEG movie or Motion JPEG */
+    "image/jpg", "JPEG-dither", /* JPEG movie or Motion JPEG */
+    "image/png", "PNG-dither", /* Netscape Server Push */
     NULL
 };
 
@@ -272,6 +273,11 @@ int ValidImageContentType(buf, ppsz_filter)
       }
       ppsz++;
    }
+   if (UtilStrICmp(psz_copy, "text/html") == 0) {
+      if (ppsz_filter != NULL) *ppsz_filter = NULL;
+      UtilFree(psz_copy);
+      return TRUE;
+   }
    UtilFree(psz_copy);
    return FALSE;
 }
@@ -351,6 +357,9 @@ int GetContentLength(buf)
          gnHeaderLen = gnStartIndex = ((&c_ptr[inc]) - buf);
          if (good_content_type && gpszBoundary != NULL) {
             gnMultipartReplace = TRUE;
+            if (debugHttp == 99 && cmdLineDumpURL && cmdLineDumpURLWithHeader) {
+               fprintf(stdout, "%s", buf);
+            }
          } else {
             ResetMultipartReplace(FALSE);
             gnPossibleMultipartReplace = FALSE;
@@ -425,6 +434,29 @@ typedef struct tagImageContentInfo {
    int header_len;
 } ImageContentInfo;
 
+/*
+ * The data in the buffer is expected to have the following structure:
+ *
+ *     HTTP/1.0 200
+ *     Content-Type: multipart/x-mixed-replace: boundary=BOUNDARYPATTERN
+ *
+ *     --BOUNDARYPATTERN
+ *     Content-Type: image/jpeg
+ *
+ *     ~~~~..JFIF.....H.H..~~..Photoshop 3.0.8BIM.~....
+ *     3~O~~~~~~.~~~~?~G~~~~~~~~,~~~a~ \~~~
+ *
+ *     --BOUNDARYPATTERN
+ *     Content-Type: image/jpeg
+ *
+ *     ~~~~..JFIF..........~~.C...........#!.$.M2.**.^C
+ *     ..................................~~
+ *
+ * Notices that the first 3 bytes in a JPEG file is ff d8 ff and
+ * the last 2 bytes are ff d9.  This is how the code can figure
+ * out the end of the JPEG file without Content-Length.
+ */
+
 static
 int GetContent(buf, buf_len, start_index, pici)
    char *buf;
@@ -455,6 +487,10 @@ int GetContent(buf, buf_len, start_index, pici)
          } else {
             if (strcmp(line_ptr, gpszBoundary) == 0) {
                found_boundary = TRUE;
+            } else if (strncmp(gpszBoundary, "----", 4) == 0 &&
+                  strcmp(line_ptr, &gpszBoundary[2]) == 0) {
+               found_boundary = TRUE;
+               UpdateBoundary(&line_ptr[2]);
             }
             seen_none_blank_line = TRUE;
          }
@@ -490,6 +526,11 @@ int GetContent(buf, buf_len, start_index, pici)
                         break;
                      }
                   }
+               } else if (psz[0] == ((char)0x89) && psz[1] == 'P' &&
+                     psz[2] == 'N' && psz[3] == 'G') {
+                  /*
+                   * Well, Content-Length better be there!
+                   */
                } else {
                   /* this stream should be abandoned */
                }
@@ -532,6 +573,8 @@ int GetContent(buf, buf_len, start_index, pici)
                         TgLoadCachedString(CSTID_DONT_KNOW_HOW_HANDLE_FRAME_CT),
                         &colon_ptr[1]);
                   fprintf(stderr, "\n");
+               } else if (pici->psz_filter == NULL) {
+                  /* content-type is text/html, pretend it's okay */
                }
                if (psz != NULL) *psz++ = ';';
             }
@@ -563,6 +606,7 @@ void ScanBody(buf, pn_buf_len)
 
          if (tmp_fname != NULL) {
             int left_over=0;
+            int saved_enable_failed_import_msgbox=GetEnableFailedImportMsgBox();
             char *dest=NULL, *src=NULL;
 
             if (gpVideoObj != NULL) {
@@ -570,12 +614,16 @@ void ScanBody(buf, pn_buf_len)
                FreeObj(gpVideoObj);
             }
             gpVideoObj = NULL;
+            if (strcmp(ici.sz_content_type, "image/png") == 0) {
+               SetEnableFailedImportMsgBox(FALSE);
+            }
             if (ImportSpecifiedFileType(tmp_fname, ici.psz_filter)) {
                gpVideoObj = topObj;
                MoveObj(gpVideoObj, drawOrigX-gpVideoObj->obbox.ltx,
                      drawOrigY-gpVideoObj->obbox.lty);
                DrawObj(drawWindow, gpVideoObj);
             }
+            SetEnableFailedImportMsgBox(saved_enable_failed_import_msgbox);
             unlink(tmp_fname);
             free(tmp_fname);
             /* now we need to shrink the buffer */
@@ -623,7 +671,12 @@ void FakeUserAgent(buf)
    char prev_agent[MAXSTRING<<1], spec[MAXSTRING<<1];
 
    if (buf != NULL && strcmp(buf, "-1") != 0) {
+      int len=0;
+
       UtilStrCpyN(spec, sizeof(spec), buf);
+      UtilTrimBlanks(spec);
+      len = strlen(spec);
+      if (len > 0 && spec[len-1] == ')') spec[len-1] = '\0';
    } else {
       *prev_agent = '\0';
       GetUserAgent(prev_agent, sizeof(prev_agent));
