@@ -1,8 +1,8 @@
 /*
  * Author:      Daniel Ratton Figueiredo <ratton@land.ufrj.br>
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,11 +29,13 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.20 2000/10/11 13:53:09 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.27 2001/09/29 21:01:13 william Exp $
  */
 
 #define _INCLUDE_FROM_TANGRAM2_C_
 
+#include <ctype.h>
+
 #include "tgifdefs.h"
 
 #include "attr.e"
@@ -52,7 +54,9 @@
 #include "rcbox.e"
 #include "select.e"
 #include "setup.e"
+#include "strtbl.e"
 #include "tangram2.e"
+#include "text.e"
 #include "util.e"
 
 #ifdef _TANGRAM_II
@@ -227,10 +231,11 @@ ExecInfo *Tangram2GetExecInfo(func_name)
 
 /* --------------- Tangram2 Internal Commands Implementation --------------- */
 
-#define MAXVARS             200
-#define MAXVARNAME          250
-#define MAXCONNECTTRIES       3
-#define PORT               6743
+#define MAXVARS                 200
+#define MAXVARNAME              250
+#define MAXCONNECTTRIES           3
+#define PORT                   6743
+#define STVARVALUEUNDEF  2147483647
 
 #define END_OF_SIMULATION     1
 #define STEP_SIMULATION       2
@@ -418,74 +423,6 @@ void ExecEndSimulator(obj_ptr, orig_cmd)
    printf("TGIF: Simulation finished.\n");
 }
 
-static int isVector( char *varname )
-{
-    char *pt = varname;
-    while( (*pt != '[') && (*pt != '\0'))
-        pt++;
-
-    return( *pt == '[' ? pt - varname : 0 );
-}
-
-/******************************************************************************/
-/* This function extracts the state var names and values from the packet that */
-/* came from markfind and fullfill the buffer attribute of simulation step    */
-/* button.                                                                    */
-/******************************************************************************/
-static void monta_buffer(char *buffer, T_Packet pack)
-{
-    int i, pos;
-    int initial_index, final_index;
-    int temp_index;
-    char temp[MAXSTRING+1];
-    char value[MAXSTRING+1];
-    char basename[MAXSTRING+1];
-
-    buffer[0] = 0;
-    for (i=0; i < pack.varcount; i++)
-    {
-        if( ( pos = isVector( pack.vars[i].name ) ) > 0 )
-        {
-            final_index = -1;
-            strncpy( basename, pack.vars[i].name, pos );
-            basename[ pos ] = 0;
-            initial_index = atoi( &pack.vars[i].name[ pos + 1 ] );
-
-            sprintf( value, "%d", pack.vars[i].value );
-
-            i++;
-            while( (strncmp( basename, pack.vars[i].name, pos ) == 0) && 
-                   (pack.vars[i].name[pos] == '[') && (i < pack.varcount) )
-            {
-                temp_index = atoi( &pack.vars[i].name[ pos + 1 ] );
-                if( (final_index == -1 && temp_index == initial_index + 1)
-                    || temp_index == final_index + 1 )
-                {
-                    final_index = temp_index;
-                    sprintf( temp, ",%d", pack.vars[i].value );
-                    strcat( value, temp );
-                    i++;
-                }
-                else
-                    break;
-            }
-            i--;
-            if( final_index != -1 )
-                sprintf( temp, "%s[%d:%d]=[%s]@", basename, initial_index,
-                         final_index, value);
-            else
-                sprintf( temp, "%s[%d]=%s@", basename, initial_index, value);
-            strcat( buffer, temp );
-        }
-        else
-        {
-            strcat( buffer, pack.vars[i].name );
-            sprintf( temp, "=%d@", pack.vars[i].value );
-            strcat( buffer, temp );
-        }
-    }
-}
-
 static int recv_packet(int soc, T_Packet *pack_in)
 {
    int  byread, status;   /* bytes read from the socket */
@@ -541,37 +478,309 @@ static int send_packet(int soc, T_Packet pack_out)
    }
 }
 
-/******************************************************************************/
-/* This function extracts the state var names and values from buffer which    */
-/* came from simulation step button and builds a packet that goes to markfind */
-/******************************************************************************/
-static void desmonta_buffer(char *buffer, T_Packet *pack)
+static int isVector( char *varname )
 {
-   int i;
-   char *token;
-   char *pt;
+    char *pt = varname;
+    while( (*pt != '[') && (*pt != '\0') )
+        pt++;
 
-   i = 0;
-   token = strtok(buffer, "@");
-   while (token != NULL) {
-        pt = token;
-        while( (*pt != '=') && (*pt != '\0') )
-            pt++;
-        *pt = '\0';
+    return( *pt == '[' ? pt - varname : 0 );
+}
 
-        strcpy( pack->vars[i].name, token );
-        /* FIXME - if necessary feedback markfind with the state vars values */
-        /* this is the right place to get them */
-        pack->vars[i].value = 0;
+static int getVecValue( char *values, int offset )
+{
+    int commas;
+    int value;
+    char *pt;
 
-        i++;
-        if( i >= MAXVARS )
-            break;
+    value  = STVARVALUEUNDEF;
+    commas = 0;
+    pt     = values;
+    /* search the first element */
+    while( *pt != 0 && !isdigit(*pt) )
+        pt++;
 
-        token = strtok(NULL, "@");
+    while( *pt != 0 && commas < offset )
+    {
+        if( *pt == ',' )
+            commas++;
+        pt++;
+    }
+
+    /* if it is not the first element we stoped at a comma */
+    if( *pt == ',' )
+        ++pt;
+
+    if( isdigit( *pt ) )
+        value = atoi( pt );
+
+    return( value );
+}
+
+static
+void AppendLinesToAttr(attr_ptr, ppsz_lines)
+   struct AttrRec *attr_ptr;
+   char **ppsz_lines;
+{
+   char **ppsz=NULL;
+   struct TextRec *text_ptr=attr_ptr->obj->detail.t;
+
+   for (ppsz=ppsz_lines; *ppsz != NULL; ppsz++) {
+      MiniLineInfo *pFirstMiniLine=NULL, *pLastMiniLine=NULL;
+
+      CreateMiniLineFromString(*ppsz, &pFirstMiniLine, &pLastMiniLine);
+      text_ptr->minilines.last->next = pFirstMiniLine;
+      pFirstMiniLine->prev = text_ptr->minilines.last;
+      text_ptr->minilines.last = pLastMiniLine;
+      text_ptr->lines++;
    }
+   RecalcTextMetrics(text_ptr, attr_ptr->obj->x, text_ptr->baseline_y);
+   UpdTextBBox(attr_ptr->obj);
+}
 
-   pack->varcount = i;
+/******************************************************************************/
+/* Checks whether pack.var[i] and pack.var[i+1] belongs to same vector and    */
+/* if var[i+1] is exactly var[i] + 1. If so, returns 1 otherwise return 0.    */
+/******************************************************************************/
+static int isNextInSameVector( int i, T_Packet pack )
+{
+    int status;
+    int ptr;
+
+    status = 0;
+    if( i < pack.varcount - 1 )
+    {
+        ptr = strlen( pack.vars[i+1].name );
+        while( ptr > 0 && pack.vars[i+1].name[ptr] != '[' )
+            ptr--;
+        if( ptr > 0 && strncmp( pack.vars[i].name, pack.vars[i+1].name, ptr ) == 0 )
+        {
+            if( atoi(&(pack.vars[i+1].name[ptr+1])) == atoi(&(pack.vars[i].name[ptr+1])) + 1 )
+                status = 1;
+        }
+    }
+    return( status );
+}
+
+
+/******************************************************************************/
+/* If varname is state variable vector name in th form objname.varname[x],    */
+/* returns x otherwise returns 0. Prior calling this function, varname should */
+/* be checked to see if it is a state vector name cause 0 is a valid index.   */
+/******************************************************************************/
+static int getIndex( char *varname )
+{
+    int status;
+    int ptr;
+
+    status = 0;
+    ptr = strlen( varname );
+    while( ptr > 0 && varname[ptr] != '[' )
+        ptr--;
+    if( ptr > 0 )
+        status = atoi(&varname[ptr+1]);
+    return( status );
+}
+
+
+static void UpdateStateVars( obj_ptr, pack )
+struct ObjRec *obj_ptr;
+T_Packet pack;
+{
+    int i, j;
+    int begin, end;
+    int length, pos;
+    char endchar;
+    char *ptr;
+    char temp[ MAXVARNAME ];
+    char line[ MAXVARNAME ];
+    char obj_name[ MAXSTRING + 1 ];
+    char var_name[ MAXSTRING + 1 ];
+    char value[ MAXSTRING + 1 ];
+    char att_name[ 15 ];
+    struct ObjRec *owner_obj=NULL, *named_obj;
+    struct ObjRec *att_owner_obj=NULL;
+    struct AttrRec *attr_ptr;
+
+    i = 0;
+    while( i < pack.varcount )
+    {
+        CleanTmpStr();
+        ptr = strchr( pack.vars[i].name, '.' );
+        if( ptr == NULL )
+        {
+            i++;
+            continue;
+        }
+        length = strlen( pack.vars[i].name ) - strlen( ptr );
+        strncpy( obj_name, pack.vars[i].name, length );
+        obj_name[ length ] = 0;
+        j = 0;
+        /* Find all state variables of this object */
+        while( i < pack.varcount && strncmp( obj_name, pack.vars[i].name, length ) == 0 )
+        {
+            /* Get state variable name */
+            pos = length + 1;
+            while( pack.vars[i].name[pos] != '[' &&
+                   pack.vars[i].name[pos] != 0 )
+                pos++;
+            endchar = pack.vars[i].name[pos];
+            strncpy( var_name, &(pack.vars[i].name[length+1]), pos - length );
+            var_name[ pos - length - 1 ] = 0;
+
+            if( endchar == '[' )
+            {
+                end = begin = getIndex( pack.vars[i].name );
+                sprintf( value, "%d", pack.vars[i].value );
+                while( isNextInSameVector( i, pack ) )
+                {
+                    if( getIndex( pack.vars[ i + 1 ].name ) == end + 1 )
+                    {
+                        end++;
+                        sprintf( temp, ",%d", pack.vars[ i + 1 ].value );
+                        strcat( value, temp );
+                        i++;
+                    }
+                    else
+                        break;
+                }
+
+                if( end == begin )
+                    sprintf( line, "%s[%d]=%s", var_name, begin, value );
+                else
+                    sprintf( line, "%s[%d:%d]=[%s]", var_name, begin, end, value );
+            }
+            else
+            {
+                sprintf( line, "%s=%d", &(pack.vars[i].name[length]) + 1, 
+                         pack.vars[i].value );
+            }
+
+            AppendToTmpStr( line );
+            j++;
+            i++;
+        }
+
+        named_obj = FindObjWithName( botObj, obj_ptr, obj_name, FALSE, FALSE, 
+                                     &owner_obj, NULL );
+        strcpy( att_name, "State_vars=" );
+        attr_ptr  = FindAttrWithName( named_obj, att_name, &att_owner_obj );
+        if( attr_ptr != NULL )
+        { 
+            char *count_buf=(char*)malloc((strlen(att_name)+40)*sizeof(char));
+
+            if( count_buf == NULL )
+                FailAllocMessage();
+            sprintf( count_buf, "%s", att_name );
+            if( PrependToTmpStr( count_buf ) )
+              ReplaceAttrAllValues(named_obj, attr_ptr, &topTmpStr, &botTmpStr);
+            free(count_buf);
+        }
+    }
+    CleanTmpStr();
+}
+
+void build_pack( obj_ptr, pack )
+struct ObjRec *obj_ptr;
+T_Packet *pack;
+{
+    int i, j, pos;
+    int numvars;
+    int need_to_free_tmp_buf;
+    int processingvarname;
+    int initial_index, final_index;
+    char varvalue[ MAXSTRING ];
+    char varname [ MAXSTRING ];
+    char *tmp_buf;
+    struct ObjRec *first_obj_ptr, *optr;
+    struct AttrRec *name_attr;
+    struct AttrRec *stat_attr;
+    MiniLineInfo *pMiniLine;
+
+    numvars = 0;
+    first_obj_ptr = topObj;
+    for( optr = first_obj_ptr; optr != NULL; optr = optr->next )
+    {
+        stat_attr = FindAttrWithName(optr,"State_vars=",NULL);
+        if( stat_attr != NULL )
+        {
+            name_attr = FindAttrWithName(optr,"name=",NULL);
+            for( pMiniLine=stat_attr->obj->detail.t->minilines.first;
+                 pMiniLine != NULL;
+                 pMiniLine=pMiniLine->next) 
+            {
+                tmp_buf = ConvertMiniLineToString(pMiniLine, &need_to_free_tmp_buf);
+
+                processingvarname = 1;
+                for( i = j = 0; tmp_buf[i] != 0; i++ )
+                {
+                    if( tmp_buf[i] != ' ' && tmp_buf[i] != '=' )
+                        if( processingvarname )
+                            varname[j++] = tmp_buf[i];
+                        else
+                            varvalue[j++] = tmp_buf[i];
+                    if( tmp_buf[i] == '=' )
+                    {
+                        processingvarname = 0;
+                        varname[ j ] = 0;
+                        j = 0;
+                    }
+                }
+                if( processingvarname )
+                    varname[ j ]  = 0;
+                else
+                    varvalue[ j ] = 0;
+
+                if( strlen(varname)>0 && strncmp(varname,"State_vars",10) != 0 )
+                {
+                    if( (pos = isVector( varname )) > 0 )
+                    {
+                        varname[ pos ] = 0;
+                        initial_index = atoi( &varname[ ++pos ] );
+
+                        /* Skip initial index */
+                        while( isdigit( varname[ pos ] ) && varname[pos] != 0 )
+                            pos++;
+
+                        switch( varname[ pos ] )
+                        {
+                            case ':': final_index = atoi(&varname[++pos]);break;
+                            case ']': final_index = initial_index;        break;
+                            default: final_index = -1;
+                        }
+
+                        for( j = initial_index; j <= final_index; j++,numvars++)
+                        {
+                            sprintf( pack->vars[numvars].name, "%s.%s[%d]",
+                                 name_attr->attr_value.s, varname, j );
+                            pack->vars[numvars].value = getVecValue( varvalue,
+                                                            j - initial_index );
+                            if( numvars >= MAXVARS )
+                                break;
+                        }
+                    }
+                    else /* isVector */
+                    {
+                        sprintf( pack->vars[numvars].name, "%s.%s",
+                                 name_attr->attr_value.s, varname );
+                        if( isdigit( varvalue[0] ) )
+                            pack->vars[ numvars ].value = atoi( varvalue );
+                        else
+                            pack->vars[ numvars ].value = STVARVALUEUNDEF;
+
+                        if( ++numvars >= MAXVARS )
+                            break;
+                    }
+                }
+
+                if( need_to_free_tmp_buf )
+                    UtilFree(tmp_buf);
+            }
+        }
+    }
+    if( numvars > MAXVARS )
+        numvars = MAXVARS;
+    pack->varcount = numvars;
 }
 
 void ExecSimulateStep(obj_ptr, orig_cmd)
@@ -599,12 +808,6 @@ void ExecSimulateStep(obj_ptr, orig_cmd)
       return;
    }
    
-   /* pega o valor do atributo buffer */
-   strcpy(execDummyStr, "buffer=");
-   attr_ptr = FindAttrWithName(obj_ptr, execDummyStr, &attr_owner_obj);
-   if (attr_ptr == NULL) {BadAttr_Simulator(execDummyStr, orig_cmd); return; }
-   strcpy(buffer, attr_ptr->attr_value.s);
-
    /* pega o valor do atributo step */
    strcpy(execDummyStr, "Step=");
    attr_ptr = FindAttrWithName(obj_ptr, execDummyStr, &attr_owner_obj);
@@ -623,8 +826,7 @@ void ExecSimulateStep(obj_ptr, orig_cmd)
    pack_out.code = STEP_SIMULATION;
    pack_out.step = step;
 
-
-   desmonta_buffer(buffer, &pack_out);
+   build_pack( obj_ptr, &pack_out);
 
    /* envia o pacote ao simulador */
    if (send_packet(simulator_socket, pack_out) < 0) {
@@ -637,13 +839,6 @@ void ExecSimulateStep(obj_ptr, orig_cmd)
    /* desmonta o pacote */
    tempo = pack_in.elapsed_time;
    transitions = step;
-   monta_buffer(buffer, pack_in);
-
-   /* seta o valor do atributo buffer  */
-   strcpy(execDummyStr, "buffer=");
-   attr_ptr = FindAttrWithName(obj_ptr, execDummyStr, &attr_owner_obj);
-   if (attr_ptr == NULL) { BadAttr_Simulator(execDummyStr, orig_cmd); return; }
-   ReplaceAttrFirstValue(attr_owner_obj, attr_ptr, buffer);
 
    /* set o valor do step_time (tempo de duracao deste step) */
    strcpy(execDummyStr, "step_time=");
@@ -671,6 +866,8 @@ void ExecSimulateStep(obj_ptr, orig_cmd)
    transitions += atof(attr_ptr->attr_value.s);
    sprintf(execDummyStr, "%d", transitions);
    ReplaceAttrFirstValue(attr_owner_obj, attr_ptr, execDummyStr);
+
+   UpdateStateVars( obj_ptr, pack_in );
 }
 
 /* ======================= Tangram2 Menu ======================= */
@@ -845,18 +1042,14 @@ static char *GenerateChainWriteObj[] = {
   "append_attr_into_file(str,$(file_name));",
   NULL
 };
-/* ----------------------------------------------------------------------- */
+/*-----------------------------------------------------------------------*/
 static char *StartSimulatorSetModeB[] = {
-/*  "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) B $(__GENERATE_CHAIN__.dbglevel) $(Event) $(Transitions) $(Time) $(Runs) >> $(__GENERATE_CHAIN__.output) & \");", */
-/*  "launch(command);", */
   NULL
 };
 
 static char *StartSimulatorSetModeI[] = {
  "strcpy(Transitions, \"0\");",
  "strcpy(Time, \"0.0\");",
-/* "strcpy(command, \"simulate_chain $(__GENERATE_CHAIN__.basename) I $(__GENERATE_CHAIN__.dbglevel) >> $(__GENERATE_CHAIN__.output) & \");", */
-/* "launch(command);", */
  "start_simulator();",
   NULL
 };
@@ -883,7 +1076,7 @@ static char *StartSimulatorExec[] = {
  "enable_undo();",
  NULL
 };
-/* ----------------------------------------------------------------------- */
+/*-----------------------------------------------------------------------*/
 static char *SimulationStepLoopAnimation[] = {
  "for_i(Ani_C_Step,1,$(Ani_T_Steps),1,Do_Animation);",
  "round(sleep_time,$(Time_Scale)*$(step_time)*1000);",
@@ -925,117 +1118,45 @@ static char *SimulationStepStop[] = {
 
 static char *SimulationStepExec[] = {
  "disable_undo();",
- /*"show_attr(Step);",
- "show_attr(Delay);",
- "show_attr(Animation);",
- "show_attr(Time_Scale);",*/
  "if(\"$(Step)\"==\"0\",play_simulation,step_simulation);",
  "enable_undo();",
  "redraw_drawing_area();",
   NULL
 };
 
-
 static char *SimulationStepPlaySimulation[] = {
   "while(1,play_one_step);",
   NULL
 };
 
 static char *SimulationStepStepSimulation[] = {
- "strcpy(buffer,\"\");",
- "find_obj_names(obj_list,\"\",\"State_vars=*\");",
- "get_line_in_attr(no_objs,obj_list,0);",
-
- "for_i(obj_index,1,$(no_objs),1,parse_obj);",
-/* aqui furou */
  "exec(call_simulator);",
- "exec(update_objects);",
- "redraw_drawing_area();",
  "if(\"$(Animation)\"==\"ON\",Loop_Animation,NULL);",
   NULL
 };
 
+/**
 static char *SimulationStepParseObj[] = {
- "get_line_in_attr(cur_obj,obj_list,$(obj_index));",
- "get_number_of_lines_in_attr(no_state_vars,$(cur_obj).State_vars);",
- "for_i(var_index,1,$(no_state_vars)-1,1,write_vars);",
-/* "for_i(var_index,1,2,1,write_vars);", */
- "clear_attr($(cur_obj).State_vars);",
   NULL
 };
 
 static char *SimulationStepWriteVars[] = {
-  /* FIXME this part must be used to feedback (state vars) to markfind */
- "get_line_in_attr(state_var_line,$(cur_obj).State_vars,$(var_index));",
- "trim(state_var_line);",
- "strlen(line_size, $(state_var_line));",
- "if($(line_size)>=1,write_var_line,NULL);",
-/* "if(1==1,write_var_line,NULL);", */
   NULL
 };
 
 static char *SimulationStepWriteVarLine[] = {
- /* wipes out the spaces inside the lines eg: S = 3 -> S=3 */
- "tokenize(state_var_tok,$(state_var_line),'=');",
- "get_line_in_attr(state_var_name,state_var_tok,1);",
- "get_line_in_attr(state_var_value,state_var_tok,2);",
- "strcpy(new_line, \"$(cur_obj).$(state_var_name)=$(state_var_value)\");",
- /* append state var assignment (without spaces) in buffer attribute */
- "strcpy(buffer,\"$(buffer)$(new_line)@\");",
-/* "strcpy(buffer,\"<obj.state_var>=<value>@\");", */
   NULL
 };
 
 static char *SimulationStepUpdateObjects[] = {
- "tokenize(state_var_list,$(buffer),'@');",
- "get_line_in_attr(no_state_vars,state_var_list,0);",
- "for_i(var_index,1,\"$(no_state_vars)-1\",1,update_vars);",
   NULL
 };
 
 static char *SimulationStepUpdateVars[] = {
- "get_line_in_attr(state_var_str,state_var_list,$(var_index));",
- "tokenize(state_var_tok,$(state_var_str),'.');",
- "get_line_in_attr(obj_name,state_var_tok,1);",
- "get_line_in_attr(state_var_str,state_var_tok,2);",
- "tokenize(state_var_tok,$(state_var_str),'=');",
- "get_line_in_attr(state_var_name,state_var_tok,1);",
- "get_line_in_attr(state_var_value,state_var_tok,2);",
- /* sets the color, font and size of the State_vars attribute */
- "select_obj_by_name($(obj_name));",
-/* "set_selected_obj_pen(0000ff);", */
-/* "set_selected_text_size(14);", */
-/* "set_selected_text_font(\"Courier\");", */
-/*"set_color(\"black\");", */
- "append_line_into_attr($(obj_name).State_vars,\" $(state_var_name)=$(state_var_value)\");",
- "show_attr($(obj_name).State_vars);",
   NULL
 };
-
-/* ----------------------------------------------------------------------- */
-
-static
-void AppendLinesToAttr(attr_ptr, ppsz_lines)
-   struct AttrRec *attr_ptr;
-   char **ppsz_lines;
-{
-   char **ppsz=NULL;
-   struct TextRec *text_ptr=attr_ptr->obj->detail.t;
-
-   for (ppsz=ppsz_lines; *ppsz != NULL; ppsz++) {
-      MiniLineInfo *pFirstMiniLine=NULL, *pLastMiniLine=NULL;
-
-      CreateMiniLineFromString(*ppsz, &pFirstMiniLine, &pLastMiniLine);
-      text_ptr->minilines.last->next = pFirstMiniLine;
-      pFirstMiniLine->prev = text_ptr->minilines.last;
-      text_ptr->minilines.last = pLastMiniLine;
-      text_ptr->lines++;
-   }
-   RecalcTextMetrics(text_ptr, attr_ptr->obj->x, text_ptr->baseline_y);
-   UpdTextBBox(attr_ptr->obj);
-}
-
-/* ======================= Tangram2 Auxiliar Functions ======================= */
+**/
+/* ====================== Tangram2 Auxiliar Functions ======================= */
 
 struct ObjRec *createGenerateChainButton( dbglevel, maxstates, output )
 char *dbglevel;
@@ -1198,9 +1319,6 @@ char *step;
    AddAttrByNameAndValue(tmp_box_obj, "state_var_tok=", "");
    AddAttrByNameAndValue(tmp_box_obj, "var_index=", "");
 
-   AddAttrByNameAndValue(tmp_box_obj, "write_var_line=", "");
-   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepWriteVarLine);
-
    AddAttrByNameAndValue(tmp_box_obj, "Loop_Animation=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepLoopAnimation);
 
@@ -1216,9 +1334,6 @@ char *step;
    AddAttrByNameAndValue(tmp_box_obj, "play_one_step=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepPlayOneStep);
 
-   AddAttrByNameAndValue(tmp_box_obj, "update_objects=", "");
-   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepUpdateObjects);
-
    AddAttrByNameAndValue(tmp_box_obj, "stop=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepStop);
 
@@ -1227,6 +1342,9 @@ char *step;
 
    AddAttrByNameAndValue(tmp_box_obj, "play_simulation=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepPlaySimulation);
+/**
+   AddAttrByNameAndValue(tmp_box_obj, "update_objects=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepUpdateObjects);
 
    AddAttrByNameAndValue(tmp_box_obj, "update_vars=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepUpdateVars);
@@ -1234,12 +1352,15 @@ char *step;
    AddAttrByNameAndValue(tmp_box_obj, "parse_obj=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepParseObj);
 
-   AddAttrByNameAndValue(tmp_box_obj, "exec=", "");
-   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepExec);
-
    AddAttrByNameAndValue(tmp_box_obj, "write_vars=", "");
    AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepWriteVars);
 
+   AddAttrByNameAndValue(tmp_box_obj, "write_var_line=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepWriteVarLine);
+**/
+   AddAttrByNameAndValue(tmp_box_obj, "exec=", "");
+   AppendLinesToAttr(tmp_box_obj->fattr, SimulationStepExec);
+
    AddNewSelObj(tmp_box_obj);
 
    return( tmp_box_obj );
@@ -1393,4 +1514,107 @@ void Tangram2EndSimulator()
    RedrawDrawWindow(botObj);
 }
 
+/* ======================= Tangram2 Misc Functions ======================= */
+
+void Tangram2UpdateInitAttr(port_obj, signal_name)
+   struct ObjRec *port_obj;
+   char *signal_name;
+{
+   struct AttrRec *attr_ptr=FindAttrWithName(port_obj, "name=", NULL);
+   MiniLinesInfo *minilines=NULL;
+   MiniLineInfo *pMiniLine=NULL;
+   struct TextRec *text_ptr=NULL;
+   char *port_name=NULL, *new_line=NULL, *parent_obj_name=NULL;
+   int port_name_len=0;
+
+   if (attr_ptr == NULL) return;
+   port_name = attr_ptr->attr_value.s;
+   port_name_len = strlen(port_name);
+   if (port_name_len == 0) {
+      MsgBox(TgLoadString(STID_PORT_OBJ_HAS_EMPTY_NAME), TOOL_NAME, INFO_MB);
+      return;
+   }
+   attr_ptr = FindAttrWithName(port_obj->tmp_parent, "type=", NULL);
+   if (attr_ptr != NULL &&
+         strcmp(attr_ptr->attr_value.s, "tgBroadcastWire") == 0) {
+      /* a tgBroadcastWire object is really not a Tangram-II object */
+      return;
+   }
+   attr_ptr = FindAttrWithName(port_obj->tmp_parent, "name=", NULL);
+   if (attr_ptr != NULL) {
+      parent_obj_name = attr_ptr->attr_value.s;
+   }
+   attr_ptr = FindAttrWithName(port_obj->tmp_parent, "Initialization=", NULL);
+   if (attr_ptr == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_FIND_INIT_ATTR_PORT_PA),
+            (parent_obj_name==NULL ? TgLoadCachedString(CSTID_PARANED_UNKNOWN) :
+            parent_obj_name));
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return;
+   }
+   text_ptr = attr_ptr->obj->detail.t;
+   minilines = (&text_ptr->minilines);
+   for (pMiniLine=minilines->first->next; pMiniLine != NULL;
+         pMiniLine=pMiniLine->next) {
+      int need_to_free_tmp_buf=FALSE, blank_count=0;
+      char *tmp_buf=ConvertMiniLineToString(pMiniLine, &need_to_free_tmp_buf);
+      char *attr_value=UtilStrDup(tmp_buf), *psz=NULL, *start=NULL;
+
+      if (attr_value == NULL) FailAllocMessage();
+      for (start=attr_value; *start == ' ' || *start == '\t'; start++) {
+         blank_count++;
+      }
+      if (*start != '\0') {
+         char *prefix=NULL;
+
+         if (blank_count > 0) {
+            char saved_ch=(*start);
+
+            *start = '\0';
+            prefix = UtilStrDup(attr_value);
+            if (prefix == NULL) FailAllocMessage();
+            *start = saved_ch;
+         }
+         psz = strchr(start, '=');
+         if (psz != NULL) {
+            *psz = '\0';
+            if (strncmp(port_name, start, port_name_len) == 0 &&
+                  (start[port_name_len] == ' ' ||
+                  start[port_name_len] == '\t' ||
+                  start[port_name_len] == '\0')) {
+               /* found it */
+               int misc_len=strlen(&start[port_name_len]);
+               int len=blank_count+port_name_len+(misc_len<<1)+1+
+                     strlen(signal_name);
+
+               new_line = (char*)malloc(len);
+               if (new_line == NULL) FailAllocMessage();
+               sprintf(new_line, "%s%s%s=%s%s",
+                     (prefix==NULL ? "" : prefix), port_name,
+                     &start[port_name_len], &start[port_name_len],
+                     signal_name);
+               break;
+            }
+            *psz = '=';
+         }
+         UtilFree(prefix);
+      }
+      UtilFree(attr_value);
+      if (need_to_free_tmp_buf) UtilFree(tmp_buf);
+   }
+   if (pMiniLine == NULL) {
+      sprintf(gszMsgBox, TgLoadString(STID_CANNOT_FIND_VAR_IN_INIT_ATTR),
+            port_name, (parent_obj_name==NULL ?
+            TgLoadCachedString(CSTID_PARANED_UNKNOWN) : parent_obj_name));
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      return;
+   }
+   FreeStrBlockList(pMiniLine->first_block->next);
+   pMiniLine->first_block->next = NULL;
+   pMiniLine->last_block = pMiniLine->first_block;
+   DynStrSet(&pMiniLine->first_block->seg->dyn_str, new_line);
+   RecalcTextMetrics(text_ptr, attr_ptr->obj->x, text_ptr->baseline_y);
+   UpdTextBBox(attr_ptr->obj);
+}
+
 #endif /* _TANGRAM_II */
