@@ -1,7 +1,7 @@
 %
-% Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+% Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
 %
-% Copyright (C) 1992-2000, William Chia-Wei Cheng.
+% Copyright (C) 1990-2001, William Chia-Wei Cheng.
 %
 % Permission limited to the use, copy, display, distribute without
 % charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
 % NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 % CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 %
-% @(#)$Header: /mm/src/tgif/v4/RCS/testdrive.pl,v 4.3 2000/01/16 15:22:29 william Exp $
+% @(#)$Header: /mm/src/tgif/v4/RCS/testdrive.pl,v 4.5 2001/08/06 21:51:39 william Exp $
 %
 
 % ?- compile(testdrive).
