@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.c,v 4.12 2000/05/20 22:17:37 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.c,v 4.14 2001/09/05 22:25:38 william Exp $
  */
 #define _INCLUDE_FROM_TGIF_DBG_C_
 
@@ -174,6 +174,9 @@ void tgif_free(pVoid)
       fprintf(gpTraceFile, "f: %6d 0x%08x\n",
             gnFreeCount, (unsigned int)pVoid);
    }
+   if (pVoid == NULL) {
+      fprintf(stderr, "ERROR: free(NULL) called!\n");
+   }
    free(pVoid);
 }
 
