@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.h,v 4.5 2000/05/20 22:06:47 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.h,v 4.7 2001/09/05 20:33:45 william Exp $
  */
 
 #ifndef _TGIF_DBG_H_
@@ -43,6 +43,12 @@
 extern void tgif_dbg_add_to_log ARGS_DECL((char*));
 extern void tgif_dbg_enable ARGS_DECL((int));
 
+extern void got_trigger ARGS_DECL((int));
+extern void tgif_track ARGS_DECL((int));
+
+extern int  gnAllocTrigger;
+extern int  gnFreeTrigger;
+
 extern void *tgif_malloc ARGS_DECL((size_t));
 extern void *tgif_realloc ARGS_DECL((void*, size_t));
 extern void tgif_free ARGS_DECL((void*));
