@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/util.c,v 4.10 2000/01/15 22:24:22 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/util.c,v 4.12 2001/08/06 21:39:17 william Exp $
  */
 
 #define _INCLUDE_FROM_UTIL_C_
@@ -408,13 +408,20 @@ int UtilCopyFile(pszFromFile, pszToFile)
    char *pszFromFile, *pszToFile;
 {
    char buf[0x1000];
-   int bytes_read;
-   int fd1, fd2;
+   int bytes_read=0;
+   int fd1=(-1), fd2=(-1);
 
    if ((fd1=open(pszFromFile, O_RDONLY)) == (-1)) return TG_REMOTE_STATUS_READ;
-   if ((fd2=open(pszToFile, O_WRONLY|O_CREAT|O_TRUNC)) == (-1)) {
-      close(fd1);
-      return TG_REMOTE_STATUS_WRITE;
+   if (UtilPathExists(pszToFile)) {
+      if ((fd2=open(pszToFile, O_WRONLY|O_TRUNC)) == (-1)) {
+         close(fd1);
+         return TG_REMOTE_STATUS_WRITE;
+      }
+   } else {
+      if ((fd2=open(pszToFile, O_WRONLY|O_CREAT|O_TRUNC)) == (-1)) {
+         close(fd1);
+         return TG_REMOTE_STATUS_WRITE;
+      }
    }
    while ((bytes_read=read(fd1, buf, sizeof(buf))) > 0) {
       if (write(fd2, buf, bytes_read) <= 0) {
