@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/version.c,v 4.8 2000/01/17 18:55:21 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/version.c,v 4.12 2001/10/16 01:30:45 william Exp $
  */
 
 #define _INCLUDE_FROM_VERSION_C_
@@ -40,16 +40,25 @@
 
 char *versionString="4.1";
 char *authorEmailString="bill.cheng@acm.org";
-char *copyrightString="Copyright (C) 1990-2000, William Chia-Wei Cheng";
+#ifdef QPL_LICENSE
+char *copyrightString="Copyright (C) 2001, William Chia-Wei Cheng";
+#else /* ~QPL_LICENSE */
+char *copyrightString="Copyright (C) 1990-2001, William Chia-Wei Cheng";
+#endif /* QPL_LICENSE */
 char *homePageURL="http://bourbon.cs.umd.edu:8001/tgif/";
-char *mailingList="kragen-tgif@kragen.dnaco.net";
-char *joinMailingList="kragen-tgif-subscribe@kragen.dnaco.net";
+char *oldMailingList="kragen-tgif@kragen.dnaco.net";
+char *mailingList="tgif4-announce@yahoogroups.com";
+char *joinMailingList="tgif4-announce-subscribe@yahoogroups.com";
 char *currentReleaseURL="http://bourbon.cs.umd.edu:8001/tgif/current.html";
 char *hyperGraphicsURL="http://bourbon.cs.umd.edu:8001/tgif/index.obj";
 
 char fullVersionString[MAXSTRING];
 char fullToolName[MAXSTRING];
+#ifdef QPL_LICENSE
+char *specialBuild="QPL";
+#else /* ~QPL_LICENSE */
 char *specialBuild="";
+#endif /* QPL_LICENSE */
 
 void SetFullVersionString()
 {
