@@ -1,7 +1,7 @@
 /*
- * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
+ * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2000, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2001, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,17 +28,23 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/wb.c,v 4.57 2000/06/07 22:33:46 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/wb.c,v 4.71 2001/10/16 20:22:01 william Exp $
  */
 
 #define _INCLUDE_FROM_WB_C_
 
+#define FOREVER while(1)
+
+#include <pthread.h>
 #include "tgifdefs.h"
+#include "cmdids.h"
 
 #ifdef _TGIF_WB2
 #ifdef _HAS_STREAMS_SUPPORT
 #include <stropts.h>
+#include <sys/types.h>
 #endif /* _HAS_STREAMS_SUPPORT */
+#include "rmcast/rmcast.h"
 #endif /* _TGIF_WB2 */
 
 #include "auxtext.e"
@@ -58,16 +64,17 @@
 #include "remote.e"
 #include "select.e"
 #include "setup.e"
-#include "strtbl.e"
 #include "stk.e"
+#include "strtbl.e"
 #include "tcp.e"
 #include "util.e"
 #include "wb.e"
 #include "wb_buff.e"
-#include "wb_mcast.e"
+#include "wb_seg.e"
 
 struct WhiteBoardRec	gstWBInfo;
 
+
 int	cmdLineNoWhiteBoard=FALSE;
 int	cmdLineWhiteBoardListenOnly=FALSE;
 
@@ -75,13 +82,14 @@ static int	recordCmdLogicalClock=0;
 static char	recordCmdSenderProcID[MAXSTRING];
 
 #ifdef _TGIF_WB2
-static struct sigaction	act_IO;
-static sigset_t		sinal;
+pthread_mutex_t  message_rcv_mutex    = PTHREAD_MUTEX_INITIALIZER;
 #ifdef _TGIF_DBG_WB2
 static int		wb2DebugLevel=0;
 #endif /* _TGIF_DBG_WB2 */
 #endif /* _TGIF_WB2 */
 
+
+
 /* =================== Remote Related Functions =================== */
 
 static
@@ -149,8 +157,9 @@ int CreateWBListenSocket()
 {
    gstWBInfo.listening = TRUE;
 
-   MkTempFile(gstWBInfo.listen_fname, sizeof(gstWBInfo.listen_fname), tmpDir,
-         TOOL_NAME);
+   sprintf(gstWBInfo.listen_fname, "%sTgifLXXXXXX", TMP_DIR);
+   mktemp(gstWBInfo.listen_fname);
+   unlink(gstWBInfo.listen_fname);
    if ((gstWBInfo.listen_fp=fopen(gstWBInfo.listen_fname, "w+")) == NULL) {
       fprintf(stderr, "Fail to create '%s' for deserialization.\n",
             gstWBInfo.listen_fname);
@@ -225,25 +234,17 @@ void PrintFullIDsOfObjects(psz_prefix)
 #endif /* _TGIF_DBG_WB2 */
 }
 
-int WBHasReadData(pn_flag, pn_retry)
-   int *pn_flag, *pn_retry;
-{
-   /* leave *pn_retry alone */
-   return (gstWBInfo.BlockRemoteCmdDepth <= 1 &&
-         buff_items(gstWBInfo.bd_commands) > 0);
-}
 
 int RecvWBData(flag, ppsz_buf, pn_buf_sz)
    int flag, *pn_buf_sz;
    char **ppsz_buf;
 {
-   *ppsz_buf = (char*)malloc(WB_PACK*sizeof(char));
-   if (*ppsz_buf == NULL) FailAllocMessage();
-   *pn_buf_sz = WB_PACK;
-
-   buff_rem(gstWBInfo.bd_commands, *ppsz_buf);
-
-   return TRUE;
+   
+   int retval;
+   
+   retval = buff_rem(gstWBInfo.bd_commands, (void**)ppsz_buf);
+   
+   return retval;
 }
 
 static
@@ -268,13 +269,34 @@ void ProcessWBInputData(buf)
    if (psz_content_type != NULL) UtilFree(psz_content_type);
 }
 
+int WBHasReadData(pn_flag, pn_retry)
+   int *pn_flag, *pn_retry;
+{
+   /* leave *pn_retry alone */
+   
+   if ((!(gstWBInfo.BlockRemoteCmdDepth <= 1)) &&  (buff_items(gstWBInfo.bd_commands) > 0) ) 
+   {
+    // fprintf(stderr,"gstWBInfo.BlockRemoteCmdDepth > 1 e existem dados no buffer. (WBHasReadData)\n");   
+    // exit(1);
+   }
+   
+   return (gstWBInfo.BlockRemoteCmdDepth <= 1 &&
+         buff_items(gstWBInfo.bd_commands) > 0);
+}
+
+
+
 void TryHandleWBInputData()
 {
    int flag=0, retry=TRUE;
+   char *buf;
+   int buf_sz;
+       
 
-   while (retry && WBHasReadData(&flag, &retry)) {
-      char *buf=NULL;
-      int buf_sz=0;
+     while (retry && WBHasReadData(&flag, &retry)) { 
+      
+      buf=NULL;
+      buf_sz=0;
 
       deserializingFile = TRUE;
 
@@ -283,35 +305,39 @@ void TryHandleWBInputData()
          FreeRemoteBuf(buf);
       }
       deserializingFile = FALSE;
-   }
+     } 
+
+
 }
 
-int SendWBData(pszWBData)
+int SendWBData(pszWBData, logical_clock)
    char *pszWBData;
+   int   logical_clock;
 {
-   int len=strlen(pszWBData);
-   struct WbMessage CmdSend;
+   int    nPackets1, i;
+   struct SegmentationPack *pack1;
+   
+#ifdef DEBUG   
+   fprintf(stderr,"Mensagem enviada:\n%s\n", pszWBData);
+#endif   
 
-   if (len+1 > sizeof(struct WbMessage)) {
-      sprintf(gszMsgBox, TgLoadString(STID_EXCEED_BUF_SIZE_LIMIT_IN_WB),
-            sizeof(struct WbMessage), len+1);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-      return FALSE;
+   pack1 = Segment( pszWBData, strlen(pszWBData), gszLocalPID, logical_clock, &nPackets1 );
+
+   for( i = 0; i < nPackets1; i++ )
+   {
+      RM_sendto(gstWBInfo.MCastSock, (char*)&pack1[i], sizeof(struct SegmentationPack));
+        
+      /* FIXME: Contorna a limitacao de buffer do UDP (aguarda 0.1 segs. entre envio de pcts) */
+        
+      /*
+	  if(i%10 == 9)
+	    usleep( 100000 );
+      */
    }
-   memset(&CmdSend, 0, sizeof(CmdSend));
-   memcpy(CmdSend.WbCmd, pszWBData, strlen(pszWBData));
-#ifdef _TGIF_DBG_WB2
-   fprintf(stderr, "SendWBData(), pid = %ld, Command size = %d\n",
-         gstWBInfo.pid, strlen(pszWBData));
-#endif /* _TGIF_DBG_WB2 */
-   PCM_sendto(gstWBInfo.MCastSock, (char*)(&CmdSend), sizeof(struct WbMessage),
-         0, (struct sockaddr *)(&PCM_sockaddr), sizeof(PCM_sockaddr));
+   free( pack1 );
 
 #ifdef _TGIF_DBG_WB2
    fprintf(stderr, "SendWBData(), pid = %ld\n", gstWBInfo.pid);
-   fprintf(stderr, "%s%s\n",
-         "========================================",
-         "========================================");
 #endif /* _TGIF_DBG_WB2 */
 
    return TRUE;
@@ -1037,11 +1063,6 @@ void CleanUpObsoletedWBCmds(stopped_cmd_ptr)
    struct CmdRec *cmd_ptr=NULL;
    int num_records=0;
 
-#ifdef _TGIF_WB2
-#ifdef _TGIF_DBG_WB2
-   DumpCmdStack("At the beginning of CleanUpObsoletedWBCmds()");
-#endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
    for (cmd_ptr=stopped_cmd_ptr->prev; cmd_ptr != NULL; cmd_ptr=cmd_ptr->prev) {
       num_records++;
    }
@@ -1062,11 +1083,6 @@ void CleanUpObsoletedWBCmds(stopped_cmd_ptr)
    }
    gstWBInfo.first_cmd = stopped_cmd_ptr;
    gstWBInfo.first_cmd->prev = NULL;
-#ifdef _TGIF_WB2
-#ifdef _TGIF_DBG_WB2
-   DumpCmdStack("At the end of CleanUpObsoletedWBCmds()");
-#endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
 }
 
 void HandleChatLine(cmd_ptr)
@@ -1183,7 +1199,6 @@ void ProcessRemoteCmd(psz_buf, buf_sz)
                lastPageNum = 1;
                InitPage();
                ShowPage();
-               SetFileModified(FALSE);
             }
          }
          for (cmd_ptr=gstWBInfo.cur_cmd; cmd_ptr != NULL;
@@ -1240,47 +1255,100 @@ void ProcessRemoteCmd(psz_buf, buf_sz)
 
 #ifdef _TGIF_WB2
 static
-void IO_SignalHandler()
-   /*
-    * This function handles the IO signal generated by packet throught socket.
-    */
+void *ReceivePacket( void *arg )
 {
-   struct WbMessage cmd_received;
-   char buf[WB_PACK], aux[WB_PACK];
-   char id[300];
- 
-#ifdef _TGIF_DBG_WB2
-   fprintf(stderr, "IO_SignalHandler(), pid = %ld\n", gstWBInfo.pid);
-#endif /* _TGIF_DBG_WB2 */
+    char *buf=NULL, *aux=NULL;
+    char id[300];
+    struct SegmentationPack pack1;
+    char *buffer1;
+    int byread;
 
-   if (PCM_recv(gstWBInfo.MCastSock, (char*)&cmd_received,
-         sizeof(struct WbMessage), 0) < 0) {
-      return ;
-   }
-   memcpy(buf, cmd_received.WbCmd, WB_PACK);
-   /*
-    * verifica se a mensagem nao eh dele proprio
-    */
-   if (strncmp(buf, "From: ", 6) != 0 || sscanf(&buf[6], "%s\r\n", id) != 1) {
-      fprintf(stderr, "%s.  %s, pid = %ld=tid=%s.\nbuf=%s.\n",
-            "Fatal error", "Cannot find the sender id in IO_SignalHandler()",
-            gstWBInfo.pid, id, buf);
-      return;
-   }
-#ifdef _TGIF_DBG_WB2
-   fprintf(stderr, "Id = %s\n", id);
-#endif /* _TGIF_DBG_WB2 */
-   if(strcmp(id, gszLocalPID) == 0) {
-      return;
-   }
-   /* Verifica se pode processar o comando agora, ou se deve armazenar */
-   if(gstWBInfo.BlockRemoteCmdDepth > 0) {
-      strcpy(aux, buf);
-      buff_ins(gstWBInfo.bd_commands, aux);
-      return;
-   }
-   ProcessWBInputData(buf);
-   FreeRemoteBuf(buf);
+
+
+    while( 1 )
+    {
+    
+        if( (byread = RM_recv(gstWBInfo.MCastSock, (char*) &pack1, sizeof(struct SegmentationPack))) > 0 )
+        {
+        
+#ifdef DEBUG0
+            fprintf(stdout, "__________# of received packet: %u of %u\n", ntohl(pack1.NumSeq), ntohl(pack1.NumPackets));
+#endif
+
+            if( (buffer1 = DeSegment( pack1, &byread )) != NULL ){
+                    buf = (char *)malloc( byread + 1 );
+                    memcpy( buf, buffer1, byread );
+	                buf[byread]= '\0';
+                    free(buffer1);
+#ifdef DEBUG0                    
+                    fprintf(stdout, "******Desegmentation OK\n");
+#endif                    
+            }
+            else
+                continue;
+        }else
+             continue;
+
+	    /*
+         * verifica se a mensagem nao eh dele proprio
+         */
+
+        
+		if (strncmp(buf, "From: ", 6) != 0 || sscanf(&buf[6], "%s\r\n", id) != 1) {
+           fprintf(stderr, "%s.  %s, pid = %ld=tid=%s.\nbuf=%s.\n",
+                 "Fatal error", "Cannot find the sender id in IO_SignalHandler()",
+                 gstWBInfo.pid, id, buf);
+           continue;
+        }
+
+        if(strcmp(id, gszLocalPID) == 0) {
+           continue;
+        }
+
+        /* Verifica se pode processar o comando agora, ou se deve armazenar */
+#ifdef DEBUG0        
+        fprintf(stderr,"Process the comand now? ");
+#endif        
+
+        if ((aux = (char *)malloc( byread + 1 )) == NULL)
+        {
+         fprintf(stderr,"Allocation error. (ReceivePacket)\n");
+         exit(1);
+        }
+        strncpy(aux, buf, byread);
+        aux[byread] = (char)0;
+        buff_ins(gstWBInfo.bd_commands, aux);
+
+        if(gstWBInfo.BlockRemoteCmdDepth > 0) {
+#ifdef DEBUG0        
+           fprintf(stderr,"no!\n");
+#endif           
+
+#ifdef DEBUG           
+           buff_show(gstWBInfo.bd_commands);
+#endif           
+           continue;
+        }
+        else
+        {
+            static char c='a';
+
+            if (write(talkToSelfFiledes[1], &c, 1) != 1) {
+               sprintf(gszMsgBox, TgLoadString(STID_WRITE_TO_SELF_PIPE_FAIL),
+                     TOOL_NAME);
+               fprintf(stderr, "%s\n", gszMsgBox);
+            }
+            if (c == 'g') {
+               c = 'a';
+            } else {
+               c++;
+            }
+#ifdef DEBUG0                
+            fprintf(stderr,"yes!\n");
+#endif        
+        }
+    }
+    return( NULL );
 }
 #endif /* _TGIF_WB2 */
 
@@ -1300,14 +1368,17 @@ void CleanUpWhiteBoard()
 int InitWhiteBoard()
 {
 #ifdef _TGIF_WB2
-   char logfile[MAXPATHLENGTH];
+   char localhost_name[31];
+   char pid[6];
+   char logfilename[MAXPATHLENGTH];
+   pthread_t tid;
 #endif /* _TGIF_WB2 */
 
    InitLocalPID(); /* initializes gszLocalPID */
 
    memset(&gstWBInfo, 0, sizeof(struct WhiteBoardRec));
 
-   gstWBInfo.BlockRemoteCmdDepth = 1;
+   gstWBInfo.BlockRemoteCmdDepth = 1; /* FIXME! deveria ser zero? */
    gstWBInfo.pid = (long)getpid();
 
 #ifdef _TGIF_WB2
@@ -1333,50 +1404,40 @@ int InitWhiteBoard()
       }
       /* fim do trecho */
 
-      MkTempFile(gstWBInfo.send_fname, sizeof(gstWBInfo.send_fname), tmpDir,
-            TOOL_NAME);
+      sprintf(gstWBInfo.send_fname, "%sTgifSXXXXXX", TMP_DIR);
+      mktemp(gstWBInfo.send_fname);
+      unlink(gstWBInfo.send_fname);
       if ((gstWBInfo.send_fp=fopen(gstWBInfo.send_fname, "w+")) == NULL) {
          fprintf(stderr, "Fail to create '%s' for serialization.\n",
                gstWBInfo.send_fname);
          exit(-1);
       }
       /* Creation of multicast socket */
-      sprintf(logfile, "%snome.txt", tmpDir);
-      PCM_initialize(logfile);
-      PCM_setoption(SM_INTERVAL,3);
+      
+      /* sprintf(logfile, "%snome.txt", TMP_DIR); */
+      
+      gethostname(localhost_name, 30);
 
-      PCM_conf_recv = 1;
-      PCM_conf_send = 1;
-      gstWBInfo.MCastSock = PCM_joinGroup("225.0.0.5", 5000);
+      strcpy(logfilename,"rmcastlog.");
+      strcat(logfilename,localhost_name);
+      strcat(logfilename,".");
+      sprintf(pid,"%d",(int)getpid());
+      strcat(logfilename,pid);
+
+      RM_initialize(0, NULL/* logfilename */);
+      
+      gstWBInfo.MCastSock = RM_joinGroup("225.0.0.5", 5000);
 
       gstWBInfo.bd_commands = buff_init (10, WB_PACK, UNSORTED, NULL );
-
-      /* Installation and setup of signal Handler para SIGPOLL */
-
-      sigemptyset(&sinal);
-
-      sigaddset(&sinal,SIGPOLL);
-
-      act_IO.sa_handler = IO_SignalHandler;
-
-      sigaction(SIGPOLL, &act_IO, NULL);
-
-#ifdef linux
-      if (fcntl(gstWBInfo.MCastSock, F_SETOWN, (long)getpid()) < 0) {
-         perror("SETOWN");
+      
+/* debugging RMCAST
+      fprintf(stderr,"buffer descriptor: %d\n", gstWBInfo.bd_commands);
+*/      
+      if( pthread_create( &tid, NULL, ReceivePacket, NULL ) != 0 )
+      {
+          fprintf( stderr, "Could create receive thread\n" );
+          exit( -1 );
       }
-      if (fcntl(gstWBInfo.MCastSock, F_SETFL, O_ASYNC) < 0) {
-         perror("SETFL");
-      }
-#endif /* linux */
-
-#ifdef I_SETSIG
-#ifndef linux
-      if (ioctl(gstWBInfo.MCastSock, I_SETSIG, S_RDNORM) < 0) perror("ioctl");
-
-      if (fcntl(gstWBInfo.MCastSock, F_SETFL, O_NONBLOCK) < 0) perror("fcntl");
-#endif /* ~linux */
-#endif /* I_SETSIG */
    }
 #ifdef _TGIF_DBG_WB2
    {
